Package: rxode2
Version: 5.1.5
Title: Facilities for Simulating from ODE-Based Models
Authors@R: c(
   person("Matthew L.","Fidler", role = c("aut", "cre"), email = "matthew.fidler@gmail.com", comment=c(ORCID="0000-0001-8538-6691")),
   person("Wenping", "Wang", role = c("aut"), email = "wwang8198@gmail.com"),
   person("Aaron", "Collier", role="ctb", comment="SUNDIALS/CVODE"),
   person("Alan", "Hindmarsh",role="ctb"),
   person("Arun", "Srinivasan", role="ctb", email="arunkumar.sriniv@gmail.com"),
   person("Ashley", "Crawford", role="ctb", comment="SUNDIALS/CVODE"),
   person("Awad H.", "Al-Mohy",role="ctb"),
   person("Bill", "Denney", email="wdenney@humanpredictions.com", role="ctb", comment=c(ORCID="0000-0002-5759-428X")),
   person("Cleve", "Moler", role="ctb"),
   person("Cody J.", "Balos", role="ctb", comment="SUNDIALS/CVODE"),
   person("Dan", "Shumaker", role="ctb", comment="SUNDIALS/CVODE"),
   person("Daniel", "Kaschek", email="daniel.kaschek@gmail.com", role="ctb"),
   person("Daniel R.", "Reynolds", role="ctb", comment="SUNDIALS/CVODE"),
   person("David", "Cooley", role="ctb"),
   person("David J.", "Gardner", role="ctb", comment="SUNDIALS/CVODE"),
   person("Drew", "Schmidt", role="ctb"),
   person("Ernst", "Hairer", role="ctb"),
   person("Gabriel", "Staples", role="ctb"),
   person("Gerhard", "Wanner", role="ctb"),
   person("Gilbert", "Stewart", role="ctb"),
   person("Goro", "Fuji",  role="ctb", email="gfuji@cpan.org"),
   person("Hadley", "Wickham", role="ctb"),
   person("Hidde", "van de Beek", role="ctb", email="hiddevdbeek@icloud.com"),
   person("Igor", "Kushnir", email="igorkuo@gmail.com", role="ctb"),
   person("Imperial College of Science, Technology and Medicine", role="cph"),
   person("Jack", "Dongarra", role="ctb"),
   person("Jacob", "Williams", role="ctb"),
   person("Jim", "Bunch", role="ctb"),
   person("Karline", "Soetaert", role="ctb"),
   person("Kevin", "Ushey", role="ctb"),
   person("Lawrence Livermore National Security", role="cph", comment="SUNDIALS/CVODE"),
   person("Linda", "Petzold", role="ctb"),
   person("Mark M.", "Baum", role="ctb"),
   person("Martin", "Maechler", role="ctb"),
   person("Matt", "Dowle", role="ctb", email="mattjdowle@gmail.com"),
   person("Matteo", "Fasiolo", email = "matteo.fasiolo@gmail.com", role = "ctb"),
   person("Melissa", "Hallow", role = "aut", email = "hallowkm@uga.edu"),
   person("Michel", "Lang", , "michellang@gmail.com", role = "ctb"),
   person("Morwenn","", role="ctb"),
   person("Nicholas J.", "Higham",role="ctb"),
   person("Omar", "Elashkar", email="omar.i.elashkar@gmail.com", role="ctb"),
   person("Peter N.", "Brown", role="ctb"),
   person("Radu", "Serban", role="ctb", comment="SUNDIALS/CVODE"),
   person("Rich", "FitzJohn", role="ctb", email="rich.fitzjohn@gmail.com"),
   person("Richard", "Upton", role = "ctb"),
   person("Roger B.", "Sidje", role="ctb"),
   person("Scott D.", "Cohen", role="ctb", comment="SUNDIALS/CVODE"),
   person("Simon", "Frost", role="ctb"),
   person("Slaven", "Peles", role="ctb", comment="SUNDIALS/CVODE"),
   person("Southern Methodist University", role="cph", comment="SUNDIALS/CVODE"),
   person("Thomas", "Petzoldt", role="ctb"),
   person("University of Maryland Baltimore County", role="cph", comment="SUNDIALS/CVODE"),
   person("Wes", "Hinsley", role="ctb"),
   person("Woodrow",  "Setzer", role="ctb"),
   person("Yu", "Feng", role="ctb"),
   person("Zufar", "Mulyukov", role="ctb", email="zufar.mulyukov@novartis.com"))
Maintainer: Matthew L. Fidler <matthew.fidler@gmail.com>
Depends: R (>= 4.1.0)
Suggests: Matrix, DT, covr, crayon, curl, digest, dplyr (>= 0.8.0),
        ggrepel, gridExtra, htmltools, knitr, learnr, microbenchmark,
        nlme, remotes, rlang, rmarkdown, scales, shiny, stringi,
        symengine, testthat, tidyr, usethis, withr, xgxr, pillar,
        tibble, units (>= 0.6-0), rsconnect, devtools, patchwork,
        nlmixr2data, lifecycle, kableExtra, pmxTools, rootSolve, arrow,
        fst, duckdb, DBI
Imports: PreciseSums (>= 0.7), Rcpp (>= 0.12.3), RcppParallel,
        backports, cli (>= 2.0.0), checkmate, ggplot2 (>= 3.4.0),
        inline, lotri (>= 1.0.4), memoise, methods, mirai, rex, sys,
        tools, utils, dparser (>= 1.3.1-12), qs2, rxode2ll, data.table
        (>= 1.12.4), vctrs, stats
Description: Facilities for running simulations from ordinary
    differential equation ('ODE') models, such as pharmacometrics and other
    compartmental models.  A compilation manager translates the ODE model
    into C, compiles it, and dynamically loads the object code into R for
    improved computational efficiency.  An event table object facilitates
    the specification of complex dosing regimens (optional) and sampling
    schedules.  NB: The use of this package requires both C and
    Fortran compilers, for details on their use with R please see
    Section 6.3, Appendix A, and Appendix D in the "R Administration and
    Installation" manual. Also the code is mostly released under GPL.  The
    'VODE' and 'LSODA' are in the public domain.  The vendored 'SUNDIALS'
    'CVODE' sources and headers are released under the BSD-3-Clause license.
    The information is available in the inst/COPYRIGHTS.
BugReports: https://github.com/nlmixr2/rxode2/issues/
NeedsCompilation: yes
VignetteBuilder: knitr
License: GPL (>= 3)
URL: https://nlmixr2.github.io/rxode2/,
        https://github.com/nlmixr2/rxode2/
Biarch: true
LinkingTo: sitmo, lotri (>= 1.0.0), PreciseSums (>= 0.7), Rcpp,
        RcppArmadillo (>= 0.9.300.2.0), BH, RcppParallel, RcppEigen (>=
        0.3.3.9.2), dparser (>= 1.3.1-12), StanHeaders (>= 2.21.0.7)
Encoding: UTF-8
LazyData: true
Language: en-US
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
Packaged: 2026-07-28 02:55:20 UTC; matt-fidler
Author: Matthew L. Fidler [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8538-6691>),
  Wenping Wang [aut],
  Aaron Collier [ctb] (SUNDIALS/CVODE),
  Alan Hindmarsh [ctb],
  Arun Srinivasan [ctb],
  Ashley Crawford [ctb] (SUNDIALS/CVODE),
  Awad H. Al-Mohy [ctb],
  Bill Denney [ctb] (ORCID: <https://orcid.org/0000-0002-5759-428X>),
  Cleve Moler [ctb],
  Cody J. Balos [ctb] (SUNDIALS/CVODE),
  Dan Shumaker [ctb] (SUNDIALS/CVODE),
  Daniel Kaschek [ctb],
  Daniel R. Reynolds [ctb] (SUNDIALS/CVODE),
  David Cooley [ctb],
  David J. Gardner [ctb] (SUNDIALS/CVODE),
  Drew Schmidt [ctb],
  Ernst Hairer [ctb],
  Gabriel Staples [ctb],
  Gerhard Wanner [ctb],
  Gilbert Stewart [ctb],
  Goro Fuji [ctb],
  Hadley Wickham [ctb],
  Hidde van de Beek [ctb],
  Igor Kushnir [ctb],
  Imperial College of Science, Technology and Medicine [cph],
  Jack Dongarra [ctb],
  Jacob Williams [ctb],
  Jim Bunch [ctb],
  Karline Soetaert [ctb],
  Kevin Ushey [ctb],
  Lawrence Livermore National Security [cph] (SUNDIALS/CVODE),
  Linda Petzold [ctb],
  Mark M. Baum [ctb],
  Martin Maechler [ctb],
  Matt Dowle [ctb],
  Matteo Fasiolo [ctb],
  Melissa Hallow [aut],
  Michel Lang [ctb],
  Morwenn [ctb],
  Nicholas J. Higham [ctb],
  Omar Elashkar [ctb],
  Peter N. Brown [ctb],
  Radu Serban [ctb] (SUNDIALS/CVODE),
  Rich FitzJohn [ctb],
  Richard Upton [ctb],
  Roger B. Sidje [ctb],
  Scott D. Cohen [ctb] (SUNDIALS/CVODE),
  Simon Frost [ctb],
  Slaven Peles [ctb] (SUNDIALS/CVODE),
  Southern Methodist University [cph] (SUNDIALS/CVODE),
  Thomas Petzoldt [ctb],
  University of Maryland Baltimore County [cph] (SUNDIALS/CVODE),
  Wes Hinsley [ctb],
  Woodrow Setzer [ctb],
  Yu Feng [ctb],
  Zufar Mulyukov [ctb]
Repository: CRAN
Date/Publication: 2026-07-28 10:00:02 UTC
