# Generated by roxygen2: do not edit by hand

S3method("$",rxEt)
S3method("$",rxSolve)
S3method("$",rxSolveCovs)
S3method("$",rxSolveOom)
S3method("$",rxSolveParams)
S3method("$",rxSolveSimType)
S3method("$",rxSymInvCholEnv)
S3method("$",rxUi)
S3method("$<-",rxEt)
S3method("$<-",rxEtPreview)
S3method("$<-",rxSolve)
S3method("$<-",rxSymInvCholEnv)
S3method("$<-",rxUi)
S3method("+",rxSolve)
S3method("+",solveRxDll)
S3method("[",rxEt)
S3method("[",rxEtPreview)
S3method("[",rxEvid)
S3method("[",rxRateDur)
S3method("[",rxSolve)
S3method("[<-",rxEtPreview)
S3method("[<-",rxEvid)
S3method("[<-",rxRateDur)
S3method("[<-",rxSolve)
S3method("[[",rxEvid)
S3method("[[",rxRateDur)
S3method("[[",rxSolve)
S3method("[[<-",rxEtPreview)
S3method("[[<-",rxSolve)
S3method("dimnames<-",rxSolve)
S3method("ini<-",default)
S3method("model<-",default)
S3method("rxode2<-","function")
S3method("rxode2<-",default)
S3method("units<-",rxEvid)
S3method(.DollarNames,rxEt)
S3method(.DollarNames,rxSolve)
S3method(.DollarNames,rxUi)
S3method(as.arrow,rxSolveOom)
S3method(as.character,rxEvid)
S3method(as.character,rxRateDur)
S3method(as.data.frame,rxEt)
S3method(as.data.frame,rxEtTran)
S3method(as.data.frame,rxEvid)
S3method(as.data.frame,rxRateDur)
S3method(as.data.frame,rxSolveOom)
S3method(as.data.table,rxEt)
S3method(as.data.table,rxSolveOom)
S3method(as.et,default)
S3method(as.function,character)
S3method(as.function,rxModelVars)
S3method(as.function,rxUi)
S3method(as.function,rxode2)
S3method(as.function,rxode2tos)
S3method(as.ini,call)
S3method(as.ini,character)
S3method(as.ini,data.frame)
S3method(as.ini,default)
S3method(as.ini,lotriFix)
S3method(as.ini,matrix)
S3method(as.model,call)
S3method(as.model,character)
S3method(as.model,default)
S3method(as.model,list)
S3method(as.rxUi,"function")
S3method(as.rxUi,default)
S3method(as.rxUi,rxModelVars)
S3method(as.rxUi,rxUi)
S3method(as.rxUi,rxode2)
S3method(as.rxUi,rxode2tos)
S3method(binomProbs,default)
S3method(c,rxEt)
S3method(c,rxEvid)
S3method(c,rxRateDur)
S3method(coef,"function")
S3method(coef,rxUi)
S3method(coef,rxode2)
S3method(confint,rxSolve)
S3method(dim,rxEt)
S3method(dim,rxSolveOom)
S3method(dimnames,rxSolve)
S3method(et,"function")
S3method(et,default)
S3method(et,rxParams)
S3method(et,rxSolve)
S3method(et,rxUi)
S3method(et,rxode2)
S3method(format,boundInits)
S3method(format,boundParams)
S3method(format,rxEvid)
S3method(format,rxRateDur)
S3method(format,rxSolveSimType)
S3method(getBaseIniSimModel,default)
S3method(getBaseSimModel,default)
S3method(getBaseSymengineModel,default)
S3method(head,rxSolveOom)
S3method(ini,default)
S3method(ini,rxUi)
S3method(meanProbs,default)
S3method(model,"function")
S3method(model,default)
S3method(model,rxModelVars)
S3method(model,rxUi)
S3method(model,rxode2)
S3method(modelExtract,"function")
S3method(modelExtract,default)
S3method(modelExtract,rxModelVars)
S3method(modelExtract,rxUi)
S3method(modelExtract,rxode2)
S3method(names,rxEt)
S3method(plot,rxSolve)
S3method(plot,rxSolveConfint1)
S3method(plot,rxSolveConfint2)
S3method(predict,"function")
S3method(predict,rxEt)
S3method(predict,rxParams)
S3method(predict,rxSolve)
S3method(predict,rxUi)
S3method(predict,rxode2)
S3method(print,boundInits)
S3method(print,boundParams)
S3method(print,rxC)
S3method(print,rxCoef)
S3method(print,rxDll)
S3method(print,rxEt)
S3method(print,rxEtPreview)
S3method(print,rxEtTran)
S3method(print,rxEvid)
S3method(print,rxHidden)
S3method(print,rxMemoryEstimate)
S3method(print,rxModelText)
S3method(print,rxModelVars)
S3method(print,rxRateDur)
S3method(print,rxSolve)
S3method(print,rxSolveCovs)
S3method(print,rxSolveInits)
S3method(print,rxSolveOom)
S3method(print,rxSolveParams)
S3method(print,rxSolveSimType)
S3method(print,rxSymInvCholEnv)
S3method(print,rxUi)
S3method(print,rxode2)
S3method(rbind,rxEt)
S3method(rbind,rxSolve)
S3method(rep,rxEt)
S3method(rxChain2,EventTable)
S3method(rxChain2,default)
S3method(rxCompile,character)
S3method(rxCompile,rxDll)
S3method(rxCompile,rxModelVars)
S3method(rxCompile,rxode2)
S3method(rxEtDispatchSolve,default)
S3method(rxEtDispatchSolve,rxode2et)
S3method(rxGetDistributionSimulationLines,cauchy)
S3method(rxGetDistributionSimulationLines,default)
S3method(rxGetDistributionSimulationLines,dnorm)
S3method(rxGetDistributionSimulationLines,norm)
S3method(rxGetDistributionSimulationLines,ordinal)
S3method(rxGetDistributionSimulationLines,rxUi)
S3method(rxGetDistributionSimulationLines,t)
S3method(rxHtml,rxSolve)
S3method(rxModelVarsS3,default)
S3method(rxModelVarsS3,rxUi)
S3method(rxParams,default)
S3method(rxParams,rxEt)
S3method(rxParams,rxSolve)
S3method(rxParams,rxode2)
S3method(rxRename,"function")
S3method(rxRename,default)
S3method(rxRename,rxUi)
S3method(rxShiny,default)
S3method(rxShiny,rxSolve)
S3method(rxSolve,"function")
S3method(rxSolve,default)
S3method(rxSolve,nlmixr2FitCore)
S3method(rxSolve,nlmixr2FitData)
S3method(rxSolve,rxSolve)
S3method(rxSolve,rxUi)
S3method(rxSolve,rxode2tos)
S3method(rxTrans,character)
S3method(rxTrans,default)
S3method(rxUdfUi,bolus)
S3method(rxUdfUi,default)
S3method(rxUdfUi,evid_)
S3method(rxUdfUi,expit)
S3method(rxUdfUi,infuse)
S3method(rxUdfUi,infuseDur)
S3method(rxUdfUi,linMod)
S3method(rxUdfUi,linMod0)
S3method(rxUdfUi,linModA)
S3method(rxUdfUi,linModA0)
S3method(rxUdfUi,linModB)
S3method(rxUdfUi,linModB0)
S3method(rxUdfUi,linModD)
S3method(rxUdfUi,linModD0)
S3method(rxUdfUi,linModM)
S3method(rxUdfUi,linModM0)
S3method(rxUdfUi,logit)
S3method(rxUdfUi,mix)
S3method(rxUdfUi,multiply)
S3method(rxUdfUi,obs)
S3method(rxUdfUi,phantom)
S3method(rxUdfUi,plogis)
S3method(rxUdfUi,replace)
S3method(rxUdfUi,ribeta)
S3method(rxUdfUi,ribinom)
S3method(rxUdfUi,ricauchy)
S3method(rxUdfUi,richisq)
S3method(rxUdfUi,riexp)
S3method(rxUdfUi,rif)
S3method(rxUdfUi,rigamma)
S3method(rxUdfUi,rigeom)
S3method(rxUdfUi,rinbinom)
S3method(rxUdfUi,rinorm)
S3method(rxUdfUi,ripois)
S3method(rxUdfUi,rit)
S3method(rxUdfUi,riunif)
S3method(rxUdfUi,riweibull)
S3method(rxUdfUi,rxbeta)
S3method(rxUdfUi,rxbinom)
S3method(rxUdfUi,rxcauchy)
S3method(rxUdfUi,rxchisq)
S3method(rxUdfUi,rxexp)
S3method(rxUdfUi,rxf)
S3method(rxUdfUi,rxgamma)
S3method(rxUdfUi,rxgeom)
S3method(rxUdfUi,rxnbinom)
S3method(rxUdfUi,rxnorm)
S3method(rxUdfUi,rxpois)
S3method(rxUdfUi,rxt)
S3method(rxUdfUi,rxunif)
S3method(rxUdfUi,rxweibull)
S3method(rxUiDeparse,default)
S3method(rxUiDeparse,lotriFix)
S3method(rxUiDeparse,rxControl)
S3method(rxUiGet,allCovs)
S3method(rxUiGet,cmtLines)
S3method(rxUiGet,covLhs)
S3method(rxUiGet,default)
S3method(rxUiGet,dvidLine)
S3method(rxUiGet,errParams)
S3method(rxUiGet,etaLhs)
S3method(rxUiGet,fun)
S3method(rxUiGet,funPartsDigest)
S3method(rxUiGet,funPrint)
S3method(rxUiGet,funTxt)
S3method(rxUiGet,ini)
S3method(rxUiGet,iniFun)
S3method(rxUiGet,interpLines)
S3method(rxUiGet,levelLhs)
S3method(rxUiGet,levels)
S3method(rxUiGet,lhsCov)
S3method(rxUiGet,lhsEta)
S3method(rxUiGet,lhsTheta)
S3method(rxUiGet,lhsVar)
S3method(rxUiGet,lstChr)
S3method(rxUiGet,md5)
S3method(rxUiGet,model)
S3method(rxUiGet,modelDesc)
S3method(rxUiGet,modelFun)
S3method(rxUiGet,muRefTable)
S3method(rxUiGet,multipleEndpoint)
S3method(rxUiGet,mvFromExpression)
S3method(rxUiGet,omega)
S3method(rxUiGet,paramsLine)
S3method(rxUiGet,props)
S3method(rxUiGet,sha1)
S3method(rxUiGet,simulationIniModel)
S3method(rxUiGet,simulationModel)
S3method(rxUiGet,simulationSigma)
S3method(rxUiGet,splitDose)
S3method(rxUiGet,splitDoseLines)
S3method(rxUiGet,state)
S3method(rxUiGet,stateDf)
S3method(rxUiGet,statePropDf)
S3method(rxUiGet,symengineModelNoPrune)
S3method(rxUiGet,symengineModelPrune)
S3method(rxUiGet,theta)
S3method(rxUiGet,thetaLhs)
S3method(rxUiGet,thetaLower)
S3method(rxUiGet,thetaUpper)
S3method(rxUiGet,varLhs)
S3method(seq,rxEt)
S3method(simulate,rxEt)
S3method(simulate,rxParams)
S3method(simulate,rxSolve)
S3method(simulate,rxode2)
S3method(solve,"function")
S3method(solve,rxEt)
S3method(solve,rxParams)
S3method(solve,rxSolve)
S3method(solve,rxUi)
S3method(solve,rxode2)
S3method(str,rxEt)
S3method(str,rxHidden)
S3method(str,rxSolve)
S3method(str,rxSymInvCholEnv)
S3method(str,rxUi)
S3method(str,rxode2)
S3method(summary,rxC)
S3method(summary,rxDll)
S3method(summary,rxSolve)
S3method(summary,rxode2)
S3method(t,rxSolve)
S3method(update,"function")
S3method(update,rxSolve)
S3method(update,rxUi)
S3method(vctrs::vec_cast,rxEt.tbl_df)
S3method(vctrs::vec_cast,tbl_df.rxEt)
S3method(vctrs::vec_ptype2,rxEt.tbl_df)
S3method(vctrs::vec_ptype2,tbl_df.rxEt)
S3method(vec_cast,data.frame.rxEt)
S3method(vec_cast,data.table.rxEt)
S3method(vec_cast,rxEt.data.frame)
S3method(vec_cast,rxEt.data.table)
S3method(vec_cast,rxEt.rxEt)
S3method(vec_cast,rxEt.tibble)
S3method(vec_cast,tibble.rxEt)
S3method(vec_proxy,rxEt)
S3method(vec_ptype2,data.frame.rxEt)
S3method(vec_ptype2,data.table.rxEt)
S3method(vec_ptype2,rxEt.data.frame)
S3method(vec_ptype2,rxEt.data.table)
S3method(vec_ptype2,rxEt.rxEt)
S3method(vec_restore,rxEt)
export("RxODE<-")
export("ini<-")
export("model<-")
export("rxode2<-")
export("rxode<-")
export(.assertRenameErrorModelLine)
export(.cbindOme)
export(.clearPipe)
export(.collectWarnings)
export(.convertId)
export(.copyUi)
export(.expandPars)
export(.extraC)
export(.extraCnow)
export(.getErrDist)
export(.getLastIdLvl)
export(.getNamedArgumentsToPredDf)
export(.getWh)
export(.handleSingleErrTypeNormOrTFoceiBase)
export(.iniGetAppendArg)
export(.iniHandleFixOrUnfix)
export(.iniHandleLine)
export(.malert)
export(.matchesLangTemplate)
export(.minfo)
export(.modelHandleModelLines)
export(.msuccess)
export(.mwarn)
export(.pipeDfObs)
export(.pipeDfSub)
export(.pipeEvents)
export(.pipeInits)
export(.pipeKeep)
export(.pipeNStud)
export(.pipeNSub)
export(.pipeOmega)
export(.pipeParams)
export(.pipeRx)
export(.pipeSigma)
export(.pipeThetaMat)
export(.quoteCallInfoLines)
export(.rstudioComplete)
export(.rxAdjoint)
export(.rxAdjointExpand)
export(.rxAdjointGrad)
export(.rxAdjointGradBuild)
export(.rxAdjointGradEval)
export(.rxAdjointGradEvalC)
export(.rxAdjointGradPop)
export(.rxAdjointMethodStiff)
export(.rxAdjointModel)
export(.rxAdjointSolve)
export(.rxAdjointSolveBuild)
export(.rxAdjointSolveEvalC)
export(.rxBlankIni)
export(.rxC)
export(.rxDelaySensJump)
export(.rxDelaySensJumpEvents)
export(.rxDelaySensJumpMap)
export(.rxDelaySensJumpMapCached)
export(.rxDiscreteAdjointBuild)
export(.rxDiscreteAdjointGrad)
export(.rxDiscreteForwardSens)
export(.rxDocTable)
export(.rxFromSE)
export(.rxGetHiBoundaryPred1AndIni)
export(.rxGetLambdaFromPred1AndIni)
export(.rxGetLowBoundaryPred1AndIni)
export(.rxGetPredictionDVTransform)
export(.rxGetPredictionF)
export(.rxGetPredictionFTransform)
export(.rxGetSeed)
export(.rxGetVarianceForErrorType)
export(.rxInjectMatExpOdes)
export(.rxIsLinCmt)
export(.rxJacobian)
export(.rxLastCompileSuccess)
export(.rxLinCmt)
export(.rxLinCmtGen)
export(.rxLinNcmt)
export(.rxMemoisedFns)
export(.rxPastBaseLinesFromEnv)
export(.rxPrune)
export(.rxRename)
export(.rxSEeqUsr)
export(.rxSens)
export(.rxSetSeed)
export(.rxToSE)
export(.rxTransInfo)
export(.rxTransform)
export(.rxWithOptions)
export(.rxWithSink)
export(.rxWithSinkBoth)
export(.rxWithWd)
export(.rxode2ptrs)
export(.s3register)
export(.solComp2)
export(.solComp3)
export(.symengineFs)
export(.toClassicEvid)
export(.udfCallFunArg)
export(.udfEnvReset)
export(.udfEnvSet)
export(.udfEnvSetUdf)
export(.udfExists)
export(.udfMd5Info)
export(.uiArg)
export(.useUtf)
export(.vecDf)
export(ELU)
export(GELU)
export(PReLU)
export(ReLU)
export(RxODE)
export(SELU)
export(Swish)
export(add.dosing)
export(add.sampling)
export(aes)
export(as.arrow)
export(as.et)
export(as.ini)
export(as.model)
export(as.rxEvid)
export(as.rxRateDur)
export(as.rxUi)
export(as_tibble.rxEt)
export(assertCompartmentExists)
export(assertCompartmentName)
export(assertCompartmentNew)
export(assertExists)
export(assertIniDf)
export(assertParameterValue)
export(assertRxLinCmt)
export(assertRxUi)
export(assertRxUiEstimatedResiduals)
export(assertRxUiIovNoCor)
export(assertRxUiMixedOnly)
export(assertRxUiMuRefOnly)
export(assertRxUiNoAutoregressive)
export(assertRxUiNoMix)
export(assertRxUiNormal)
export(assertRxUiPopulationOnly)
export(assertRxUiPrediction)
export(assertRxUiRandomOnIdOnly)
export(assertRxUiSingleEndpoint)
export(assertRxUiTransformNormal)
export(assertRxUnbounded)
export(assertVariableExists)
export(assertVariableName)
export(assertVariableNew)
export(binomProbs)
export(bolus)
export(boxCox)
export(boxCoxInv)
export(cvPost)
export(d2ELU)
export(d2ELUa)
export(d2GELU)
export(d2aELU)
export(d3GELU)
export(d4GELU)
export(dELU)
export(dELUa)
export(dGELU)
export(dPReLU)
export(dPReLUa)
export(dPReLUa1)
export(dReLU)
export(dSELU)
export(dSwish)
export(delay)
export(dfWishart)
export(dlReLU)
export(dmexpit)
export(dsoftplus)
export(erf)
export(et)
export(etExpand)
export(etRbind)
export(etRep)
export(etSeq)
export(etTrans)
export(eventTable)
export(evid_)
export(expand_limits)
export(expit)
export(facet_wrap)
export(findLhs)
export(forderForceBase)
export(gammap)
export(gammapDer)
export(gammapInv)
export(gammapInva)
export(gammaq)
export(gammaqInv)
export(gammaqInva)
export(genShinyApp.template)
export(geom_amt)
export(geom_cens)
export(geom_line)
export(getBaseIniSimModel)
export(getBaseSimModel)
export(getBaseSymengineModel)
export(getRxThreads)
export(ggplot)
export(guide_none)
export(indLin)
export(infuse)
export(infuseDur)
export(ini)
export(invWR1d)
export(is.rxEt)
export(is.rxSolve)
export(is.rxStackData)
export(lReLU)
export(label_both)
export(label_context)
export(label_value)
export(label_wrap_gen)
export(linMod)
export(linMod0)
export(linModA0)
export(linModB)
export(linModB0)
export(linModD)
export(linModD0)
export(linModM)
export(linModM0)
export(linToOde)
export(llikBeta)
export(llikBinom)
export(llikCauchy)
export(llikChisq)
export(llikExp)
export(llikF)
export(llikGamma)
export(llikGeom)
export(llikNbinom)
export(llikNbinomMu)
export(llikNorm)
export(llikPois)
export(llikT)
export(llikUnif)
export(llikWeibull)
export(logit)
export(logitNormInfo)
export(lotri)
export(lowergamma)
export(meanProbs)
export(mexpit)
export(mix)
export(mlogit)
export(model)
export(modelExtract)
export(obs)
export(odeMethodToInt)
export(odeToLin)
export(phantom)
export(phi)
export(probit)
export(probitInv)
export(probitNormInfo)
export(rLKJ1)
export(rinvchisq)
export(rxAllowUnload)
export(rxAppendModel)
export(rxAssignControlValue)
export(rxAssignPtr)
export(rxBlockZeros)
export(rxC)
export(rxCat)
export(rxCbindStudyIndividual)
export(rxChain)
export(rxChain2)
export(rxClean)
export(rxCombineErrorLines)
export(rxCompile)
export(rxCondition)
export(rxControl)
export(rxControlUpdateSens)
export(rxCores)
export(rxCreateCache)
export(rxD)
export(rxDelete)
export(rxDemoteAddErr)
export(rxDerived)
export(rxDeserialize)
export(rxDfdy)
export(rxDll)
export(rxDllLoaded)
export(rxDynLoad)
export(rxDynUnload)
export(rxErrTypeCombine)
export(rxEtDispatchSolve)
export(rxEventSensDeactivate)
export(rxEventSensLoadModel)
export(rxEventTableFile)
export(rxEvid)
export(rxExpandFEta_)
export(rxExpandGrid)
export(rxExpandGrid_)
export(rxExpandIfElse)
export(rxExpandSens2_)
export(rxExpandSens3_)
export(rxExpandSens_)
export(rxFixPop)
export(rxFixRes)
export(rxForget)
export(rxFromSE)
export(rxFun)
export(rxFunParse)
export(rxGetControl)
export(rxGetDefaultSerialize)
export(rxGetDistributionSimulationLines)
export(rxGetLin)
export(rxGetModel)
export(rxGetSeed)
export(rxGetSerialType_)
export(rxGetrxode2)
export(rxHasAr)
export(rxHtml)
export(rxIndLinState)
export(rxIndLinStrategy)
export(rxInit)
export(rxInits)
export(rxIntToBase)
export(rxIntToLetter)
export(rxInv)
export(rxIs)
export(rxIsAutoSwitch)
export(rxIsCurrent)
export(rxIsDense)
export(rxIsImplicit)
export(rxIsLoaded)
export(rxIsNonStiff)
export(rxIsStiff)
export(rxLastCompile)
export(rxLhs)
export(rxLoad)
export(rxLoadState)
export(rxLock)
export(rxMd5)
export(rxMemSummary)
export(rxMemoryEstimate)
export(rxModelVars)
export(rxModelVarsS3)
export(rxModels_)
export(rxNorm)
export(rxNumLoaded)
export(rxOdeToIndLin)
export(rxOldQsDes)
export(rxOmegaVarCovDeriv)
export(rxOmegaVarCovDeriv_)
export(rxOptExpr)
export(rxParam)
export(rxParams)
export(rxParseErr)
export(rxParsePk)
export(rxParsePred)
export(rxParseSetSilentErr)
export(rxParseSuppressMsg)
export(rxPkg)
export(rxPp)
export(rxPreferredDistributionName)
export(rxProgress)
export(rxProgressAbort)
export(rxProgressStop)
export(rxPrune)
export(rxRateDur)
export(rxRawToC)
export(rxReload)
export(rxRemoveControl)
export(rxRename)
export(rxRepR0_)
export(rxReq)
export(rxRmFun)
export(rxRmFunParse)
export(rxRmvn)
export(rxS)
export(rxSaveState)
export(rxSeedEng)
export(rxSensMatExp)
export(rxSerialize)
export(rxSetControl)
export(rxSetCovariateNamesForPiping)
export(rxSetIni0)
export(rxSetPipingAuto)
export(rxSetProd)
export(rxSetProgressBar)
export(rxSetSeed)
export(rxSetSilentErr)
export(rxSetSum)
export(rxSetupIni)
export(rxSetupScale)
export(rxShiny)
export(rxSimThetaOmega)
export(rxSolve)
export(rxSolveAdjoint)
export(rxSolveAdjointRk4)
export(rxSolveChunked)
export(rxSolveFree)
export(rxSolveSetup)
export(rxSplitPlusQ)
export(rxStack)
export(rxState)
export(rxStateOde)
export(rxSumProdModel)
export(rxSupportedFuns)
export(rxSuppressMsg)
export(rxSymInvChol)
export(rxSymInvCholCreate)
export(rxSymInvCholN)
export(rxSyncOptions)
export(rxTempDir)
export(rxTest)
export(rxTheme)
export(rxTick)
export(rxToIndLin)
export(rxToSE)
export(rxTrans)
export(rxUdfUi)
export(rxUdfUiControl)
export(rxUdfUiData)
export(rxUdfUiEst)
export(rxUdfUiExpr)
export(rxUdfUiFlag)
export(rxUdfUiIniDf)
export(rxUdfUiIniLhs)
export(rxUdfUiIsValue)
export(rxUdfUiMv)
export(rxUdfUiNum)
export(rxUdfUiParsing)
export(rxUdfUiReset)
export(rxUiCompress)
export(rxUiDecompress)
export(rxUiDeparse)
export(rxUiDevelop)
export(rxUiGet)
export(rxUnload)
export(rxUnloadAll)
export(rxUnlock)
export(rxUse)
export(rxValidate)
export(rxVersion)
export(rxWithPreserveSeed)
export(rxWithSeed)
export(rxbeta)
export(rxbinom)
export(rxcauchy)
export(rxchisq)
export(rxexp)
export(rxf)
export(rxgamma)
export(rxgeom)
export(rxnbinom)
export(rxnbinomMu)
export(rxnorm)
export(rxnormV)
export(rxode)
export(rxode2)
export(rxode2parse)
export(rxode2parseAssignPackagesToLoad)
export(rxode2parseAssignPointerTranslation)
export(rxode2parseAssignTranslation)
export(rxode2parseD)
export(rxode2parseGetPackagesToLoad)
export(rxode2parseGetPointerAssignment)
export(rxode2parseGetTranslation)
export(rxord)
export(rxpois)
export(rxt)
export(rxunif)
export(rxweibull)
export(scale_type)
export(scale_x_continuous)
export(scale_x_date)
export(scale_x_discrete)
export(scale_y_continuous)
export(scale_y_date)
export(scale_y_discrete)
export(setCvodeLinearSolver)
export(setRxThreads)
export(softplus)
export(splitBolus)
export(stat_amt)
export(stat_cens)
export(swapMatListWithCube)
export(testCompartmentExists)
export(testExists)
export(testIniDf)
export(testRxLinCmt)
export(testRxUnbounded)
export(testVariableExists)
export(toTrialDuration)
export(uppergamma)
export(waiver)
export(warnRxBounded)
export(write.template.server)
export(write.template.ui)
export(xlab)
export(yeoJohnson)
export(yeoJohnsonInv)
export(ylab)
export(zeroRe)
import(data.table)
import(tools)
importFrom(PreciseSums,fsum)
importFrom(Rcpp,evalCpp)
importFrom(Rcpp,sourceCpp)
importFrom(RcppParallel,RcppParallelLibs)
importFrom(checkmate,qassert)
importFrom(dparser,dparse)
importFrom(ggplot2,"%+replace%")
importFrom(ggplot2,aes)
importFrom(ggplot2,expand_limits)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,label_both)
importFrom(ggplot2,label_context)
importFrom(ggplot2,label_value)
importFrom(ggplot2,label_wrap_gen)
importFrom(ggplot2,scale_type)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_x_date)
importFrom(ggplot2,scale_x_discrete)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,scale_y_date)
importFrom(ggplot2,scale_y_discrete)
importFrom(ggplot2,waiver)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(lotri,lotri)
importFrom(memoise,is.memoised)
importFrom(memoise,memoise)
importFrom(methods,is)
importFrom(methods,signature)
importFrom(stats,ave)
importFrom(stats,coef)
importFrom(stats,dnorm)
importFrom(stats,end)
importFrom(stats,integrate)
importFrom(stats,setNames)
importFrom(stats,simulate)
importFrom(stats,start)
importFrom(stats,update)
importFrom(sys,exec_internal)
importFrom(utils,.DollarNames)
importFrom(utils,assignInMyNamespace)
importFrom(utils,capture.output)
importFrom(utils,compareVersion)
importFrom(utils,download.file)
importFrom(utils,getFromNamespace)
importFrom(utils,head)
importFrom(utils,packageVersion)
importFrom(utils,removeSource)
importFrom(utils,sessionInfo)
importFrom(utils,str)
importFrom(utils,write.csv)
importFrom(vctrs,vec_cast)
importFrom(vctrs,vec_proxy)
importFrom(vctrs,vec_ptype2)
importFrom(vctrs,vec_restore)
useDynLib(rxode2, .registration=TRUE)
