#' A list and description of rxode2 supported reserved keywords
#'
#' @format A data frame with 3 columns and 31 rows
#' \describe{
#' \item{Reserved Name}{Reserved Keyword Name}
#' \item{Meaning}{Reserved Keyword Meaning}
#' \item{Alias}{Keyword Alias}
#' }
"rxReservedKeywords"
