# This file is generated by .generateRandomUiFuns() in build.R
## nocov start
.rxnorm <- function(mean = 0, sd = 1) {
    .mean <- as.character(substitute(mean))
    .dp <- deparse1(substitute(mean))
    .tmp <- suppressWarnings(try(force(mean), silent = TRUE))
    .mean <- .uiArg(.mean, .tmp, .dp)
    .sd <- as.character(substitute(sd))
    .dp <- deparse1(substitute(sd))
    .tmp <- suppressWarnings(try(force(sd), silent = TRUE))
    .sd <- .uiArg(.sd, .tmp, .dp)
    list(replace = paste0("rxnorm(", .mean, ", ", .sd, ")"))
}

#'@export
rxUdfUi.rxnorm <- rxUdfUi.rxpois

## nocov end
# This file is generated by .generateRandomUiFuns() in build.R
## nocov start
.rxpois <- function(lambda) {
    .lambda <- as.character(substitute(lambda))
    .dp <- deparse1(substitute(lambda))
    .tmp <- suppressWarnings(try(force(lambda), silent = TRUE))
    .lambda <- .uiArg(.lambda, .tmp, .dp)
    list(replace = paste0("rxpois(", .lambda, ")"))
}

#'@export
rxUdfUi.rxpois <- rxUdfUi.rxpois

## nocov end
# This file is generated by .generateRandomUiFuns() in build.R
## nocov start
.rxt <- function(df) {
    .df <- as.character(substitute(df))
    .dp <- deparse1(substitute(df))
    .tmp <- suppressWarnings(try(force(df), silent = TRUE))
    .df <- .uiArg(.df, .tmp, .dp)
    list(replace = paste0("rxt(", .df, ")"))
}

#'@export
rxUdfUi.rxt <- rxUdfUi.rxpois

## nocov end
# This file is generated by .generateRandomUiFuns() in build.R
## nocov start
.rxunif <- function(min = 0, max = 1) {
    .min <- as.character(substitute(min))
    .dp <- deparse1(substitute(min))
    .tmp <- suppressWarnings(try(force(min), silent = TRUE))
    .min <- .uiArg(.min, .tmp, .dp)
    .max <- as.character(substitute(max))
    .dp <- deparse1(substitute(max))
    .tmp <- suppressWarnings(try(force(max), silent = TRUE))
    .max <- .uiArg(.max, .tmp, .dp)
    list(replace = paste0("rxunif(", .min, ", ", .max, ")"))
}

#'@export
rxUdfUi.rxunif <- rxUdfUi.rxpois

## nocov end
# This file is generated by .generateRandomUiFuns() in build.R
## nocov start
.rxweibull <- function(shape, scale = 1) {
    .shape <- as.character(substitute(shape))
    .dp <- deparse1(substitute(shape))
    .tmp <- suppressWarnings(try(force(shape), silent = TRUE))
    .shape <- .uiArg(.shape, .tmp, .dp)
    .scale <- as.character(substitute(scale))
    .dp <- deparse1(substitute(scale))
    .tmp <- suppressWarnings(try(force(scale), silent = TRUE))
    .scale <- .uiArg(.scale, .tmp, .dp)
    list(replace = paste0("rxweibull(", .shape, ", ", .scale, 
        ")"))
}

#'@export
rxUdfUi.rxweibull <- rxUdfUi.rxpois

## nocov end
# This file is generated by .generateRandomUiFuns() in build.R
## nocov start
.rxgeom <- function(prob) {
    .prob <- as.character(substitute(prob))
    .dp <- deparse1(substitute(prob))
    .tmp <- suppressWarnings(try(force(prob), silent = TRUE))
    .prob <- .uiArg(.prob, .tmp, .dp)
    list(replace = paste0("rxgeom(", .prob, ")"))
}

#'@export
rxUdfUi.rxgeom <- rxUdfUi.rxpois

## nocov end
# This file is generated by .generateRandomUiFuns() in build.R
## nocov start
.rxbeta <- function(shape1, shape2) {
    .shape1 <- as.character(substitute(shape1))
    .dp <- deparse1(substitute(shape1))
    .tmp <- suppressWarnings(try(force(shape1), silent = TRUE))
    .shape1 <- .uiArg(.shape1, .tmp, .dp)
    .shape2 <- as.character(substitute(shape2))
    .dp <- deparse1(substitute(shape2))
    .tmp <- suppressWarnings(try(force(shape2), silent = TRUE))
    .shape2 <- .uiArg(.shape2, .tmp, .dp)
    list(replace = paste0("rxbeta(", .shape1, ", ", .shape2, 
        ")"))
}

#'@export
rxUdfUi.rxbeta <- rxUdfUi.rxpois

## nocov end
# This file is generated by .generateRandomUiFuns() in build.R
## nocov start
.rxgamma <- function(shape, rate = 1) {
    .shape <- as.character(substitute(shape))
    .dp <- deparse1(substitute(shape))
    .tmp <- suppressWarnings(try(force(shape), silent = TRUE))
    .shape <- .uiArg(.shape, .tmp, .dp)
    .rate <- as.character(substitute(rate))
    .dp <- deparse1(substitute(rate))
    .tmp <- suppressWarnings(try(force(rate), silent = TRUE))
    .rate <- .uiArg(.rate, .tmp, .dp)
    list(replace = paste0("rxgamma(", .shape, ", ", .rate, ")"))
}

#'@export
rxUdfUi.rxgamma <- rxUdfUi.rxpois

## nocov end
# This file is generated by .generateRandomUiFuns() in build.R
## nocov start
.rxf <- function(df1, df2) {
    .df1 <- as.character(substitute(df1))
    .dp <- deparse1(substitute(df1))
    .tmp <- suppressWarnings(try(force(df1), silent = TRUE))
    .df1 <- .uiArg(.df1, .tmp, .dp)
    .df2 <- as.character(substitute(df2))
    .dp <- deparse1(substitute(df2))
    .tmp <- suppressWarnings(try(force(df2), silent = TRUE))
    .df2 <- .uiArg(.df2, .tmp, .dp)
    list(replace = paste0("rxf(", .df1, ", ", .df2, ")"))
}

#'@export
rxUdfUi.rxf <- rxUdfUi.rxpois

## nocov end
# This file is generated by .generateRandomUiFuns() in build.R
## nocov start
.rxexp <- function(rate) {
    .rate <- as.character(substitute(rate))
    .dp <- deparse1(substitute(rate))
    .tmp <- suppressWarnings(try(force(rate), silent = TRUE))
    .rate <- .uiArg(.rate, .tmp, .dp)
    list(replace = paste0("rxexp(", .rate, ")"))
}

#'@export
rxUdfUi.rxexp <- rxUdfUi.rxpois

## nocov end
# This file is generated by .generateRandomUiFuns() in build.R
## nocov start
.rxchisq <- function(df) {
    .df <- as.character(substitute(df))
    .dp <- deparse1(substitute(df))
    .tmp <- suppressWarnings(try(force(df), silent = TRUE))
    .df <- .uiArg(.df, .tmp, .dp)
    list(replace = paste0("rxchisq(", .df, ")"))
}

#'@export
rxUdfUi.rxchisq <- rxUdfUi.rxpois

## nocov end
# This file is generated by .generateRandomUiFuns() in build.R
## nocov start
.rxcauchy <- function(location = 0, scale = 1) {
    .location <- as.character(substitute(location))
    .dp <- deparse1(substitute(location))
    .tmp <- suppressWarnings(try(force(location), silent = TRUE))
    .location <- .uiArg(.location, .tmp, .dp)
    .scale <- as.character(substitute(scale))
    .dp <- deparse1(substitute(scale))
    .tmp <- suppressWarnings(try(force(scale), silent = TRUE))
    .scale <- .uiArg(.scale, .tmp, .dp)
    list(replace = paste0("rxcauchy(", .location, ", ", .scale, 
        ")"))
}

#'@export
rxUdfUi.rxcauchy <- rxUdfUi.rxpois

## nocov end
# This file is generated by .generateRandomUiFuns() in build.R
## nocov start
.rxbinom <- function(size, prob) {
    .size <- as.character(substitute(size))
    .dp <- deparse1(substitute(size))
    .tmp <- suppressWarnings(try(force(size), silent = TRUE))
    .size <- .uiArg(.size, .tmp, .dp)
    .prob <- as.character(substitute(prob))
    .dp <- deparse1(substitute(prob))
    .tmp <- suppressWarnings(try(force(prob), silent = TRUE))
    .prob <- .uiArg(.prob, .tmp, .dp)
    list(replace = paste0("rxbinom(", .size, ", ", .prob, ")"))
}

#'@export
rxUdfUi.rxbinom <- rxUdfUi.rxpois

## nocov end
# This file is generated by .generateRandomUiFuns() in build.R
## nocov start
.rinorm <- function(mean = 0, sd = 1) {
    .mean <- as.character(substitute(mean))
    .dp <- deparse1(substitute(mean))
    .tmp <- suppressWarnings(try(force(mean), silent = TRUE))
    .mean <- .uiArg(.mean, .tmp, .dp)
    .sd <- as.character(substitute(sd))
    .dp <- deparse1(substitute(sd))
    .tmp <- suppressWarnings(try(force(sd), silent = TRUE))
    .sd <- .uiArg(.sd, .tmp, .dp)
    list(replace = paste0("rinorm(", .mean, ", ", .sd, ")"))
}

#'@export
rxUdfUi.rinorm <- rxUdfUi.rxpois

## nocov end
# This file is generated by .generateRandomUiFuns() in build.R
## nocov start
.ripois <- function(lambda) {
    .lambda <- as.character(substitute(lambda))
    .dp <- deparse1(substitute(lambda))
    .tmp <- suppressWarnings(try(force(lambda), silent = TRUE))
    .lambda <- .uiArg(.lambda, .tmp, .dp)
    list(replace = paste0("ripois(", .lambda, ")"))
}

#'@export
rxUdfUi.ripois <- rxUdfUi.rxpois

## nocov end
# This file is generated by .generateRandomUiFuns() in build.R
## nocov start
.rit <- function(df) {
    .df <- as.character(substitute(df))
    .dp <- deparse1(substitute(df))
    .tmp <- suppressWarnings(try(force(df), silent = TRUE))
    .df <- .uiArg(.df, .tmp, .dp)
    list(replace = paste0("rit(", .df, ")"))
}

#'@export
rxUdfUi.rit <- rxUdfUi.rxpois

## nocov end
# This file is generated by .generateRandomUiFuns() in build.R
## nocov start
.riunif <- function(min = 0, max = 1) {
    .min <- as.character(substitute(min))
    .dp <- deparse1(substitute(min))
    .tmp <- suppressWarnings(try(force(min), silent = TRUE))
    .min <- .uiArg(.min, .tmp, .dp)
    .max <- as.character(substitute(max))
    .dp <- deparse1(substitute(max))
    .tmp <- suppressWarnings(try(force(max), silent = TRUE))
    .max <- .uiArg(.max, .tmp, .dp)
    list(replace = paste0("riunif(", .min, ", ", .max, ")"))
}

#'@export
rxUdfUi.riunif <- rxUdfUi.rxpois

## nocov end
# This file is generated by .generateRandomUiFuns() in build.R
## nocov start
.riweibull <- function(shape, scale = 1) {
    .shape <- as.character(substitute(shape))
    .dp <- deparse1(substitute(shape))
    .tmp <- suppressWarnings(try(force(shape), silent = TRUE))
    .shape <- .uiArg(.shape, .tmp, .dp)
    .scale <- as.character(substitute(scale))
    .dp <- deparse1(substitute(scale))
    .tmp <- suppressWarnings(try(force(scale), silent = TRUE))
    .scale <- .uiArg(.scale, .tmp, .dp)
    list(replace = paste0("riweibull(", .shape, ", ", .scale, 
        ")"))
}

#'@export
rxUdfUi.riweibull <- rxUdfUi.rxpois

## nocov end
# This file is generated by .generateRandomUiFuns() in build.R
## nocov start
.rigeom <- function(prob) {
    .prob <- as.character(substitute(prob))
    .dp <- deparse1(substitute(prob))
    .tmp <- suppressWarnings(try(force(prob), silent = TRUE))
    .prob <- .uiArg(.prob, .tmp, .dp)
    list(replace = paste0("rigeom(", .prob, ")"))
}

#'@export
rxUdfUi.rigeom <- rxUdfUi.rxpois

## nocov end
# This file is generated by .generateRandomUiFuns() in build.R
## nocov start
.ribeta <- function(shape1, shape2) {
    .shape1 <- as.character(substitute(shape1))
    .dp <- deparse1(substitute(shape1))
    .tmp <- suppressWarnings(try(force(shape1), silent = TRUE))
    .shape1 <- .uiArg(.shape1, .tmp, .dp)
    .shape2 <- as.character(substitute(shape2))
    .dp <- deparse1(substitute(shape2))
    .tmp <- suppressWarnings(try(force(shape2), silent = TRUE))
    .shape2 <- .uiArg(.shape2, .tmp, .dp)
    list(replace = paste0("ribeta(", .shape1, ", ", .shape2, 
        ")"))
}

#'@export
rxUdfUi.ribeta <- rxUdfUi.rxpois

## nocov end
# This file is generated by .generateRandomUiFuns() in build.R
## nocov start
.rigamma <- function(shape, rate = 1) {
    .shape <- as.character(substitute(shape))
    .dp <- deparse1(substitute(shape))
    .tmp <- suppressWarnings(try(force(shape), silent = TRUE))
    .shape <- .uiArg(.shape, .tmp, .dp)
    .rate <- as.character(substitute(rate))
    .dp <- deparse1(substitute(rate))
    .tmp <- suppressWarnings(try(force(rate), silent = TRUE))
    .rate <- .uiArg(.rate, .tmp, .dp)
    list(replace = paste0("rigamma(", .shape, ", ", .rate, ")"))
}

#'@export
rxUdfUi.rigamma <- rxUdfUi.rxpois

## nocov end
# This file is generated by .generateRandomUiFuns() in build.R
## nocov start
.rif <- function(df1, df2) {
    .df1 <- as.character(substitute(df1))
    .dp <- deparse1(substitute(df1))
    .tmp <- suppressWarnings(try(force(df1), silent = TRUE))
    .df1 <- .uiArg(.df1, .tmp, .dp)
    .df2 <- as.character(substitute(df2))
    .dp <- deparse1(substitute(df2))
    .tmp <- suppressWarnings(try(force(df2), silent = TRUE))
    .df2 <- .uiArg(.df2, .tmp, .dp)
    list(replace = paste0("rif(", .df1, ", ", .df2, ")"))
}

#'@export
rxUdfUi.rif <- rxUdfUi.rxpois

## nocov end
# This file is generated by .generateRandomUiFuns() in build.R
## nocov start
.riexp <- function(rate) {
    .rate <- as.character(substitute(rate))
    .dp <- deparse1(substitute(rate))
    .tmp <- suppressWarnings(try(force(rate), silent = TRUE))
    .rate <- .uiArg(.rate, .tmp, .dp)
    list(replace = paste0("riexp(", .rate, ")"))
}

#'@export
rxUdfUi.riexp <- rxUdfUi.rxpois

## nocov end
# This file is generated by .generateRandomUiFuns() in build.R
## nocov start
.richisq <- function(df) {
    .df <- as.character(substitute(df))
    .dp <- deparse1(substitute(df))
    .tmp <- suppressWarnings(try(force(df), silent = TRUE))
    .df <- .uiArg(.df, .tmp, .dp)
    list(replace = paste0("richisq(", .df, ")"))
}

#'@export
rxUdfUi.richisq <- rxUdfUi.rxpois

## nocov end
# This file is generated by .generateRandomUiFuns() in build.R
## nocov start
.ricauchy <- function(location = 0, scale = 1) {
    .location <- as.character(substitute(location))
    .dp <- deparse1(substitute(location))
    .tmp <- suppressWarnings(try(force(location), silent = TRUE))
    .location <- .uiArg(.location, .tmp, .dp)
    .scale <- as.character(substitute(scale))
    .dp <- deparse1(substitute(scale))
    .tmp <- suppressWarnings(try(force(scale), silent = TRUE))
    .scale <- .uiArg(.scale, .tmp, .dp)
    list(replace = paste0("ricauchy(", .location, ", ", .scale, 
        ")"))
}

#'@export
rxUdfUi.ricauchy <- rxUdfUi.rxpois

## nocov end
# This file is generated by .generateRandomUiFuns() in build.R
## nocov start
.ribinom <- function(size, prob) {
    .size <- as.character(substitute(size))
    .dp <- deparse1(substitute(size))
    .tmp <- suppressWarnings(try(force(size), silent = TRUE))
    .size <- .uiArg(.size, .tmp, .dp)
    .prob <- as.character(substitute(prob))
    .dp <- deparse1(substitute(prob))
    .tmp <- suppressWarnings(try(force(prob), silent = TRUE))
    .prob <- .uiArg(.prob, .tmp, .dp)
    list(replace = paste0("ribinom(", .size, ", ", .prob, ")"))
}

#'@export
rxUdfUi.ribinom <- rxUdfUi.rxpois

## nocov end
# This file is generated by .generateRandomUiFuns() in build.R
## nocov start
.logit <- function(x, low = 0, high = 1) {
    .x <- as.character(substitute(x))
    .dp <- deparse1(substitute(x))
    .tmp <- suppressWarnings(try(force(x), silent = TRUE))
    .x <- .uiArg(.x, .tmp, .dp)
    .low <- as.character(substitute(low))
    .dp <- deparse1(substitute(low))
    .tmp <- suppressWarnings(try(force(low), silent = TRUE))
    .low <- .uiArg(.low, .tmp, .dp)
    .high <- as.character(substitute(high))
    .dp <- deparse1(substitute(high))
    .tmp <- suppressWarnings(try(force(high), silent = TRUE))
    .high <- .uiArg(.high, .tmp, .dp)
    list(replace = paste0("logit(", .x, ", ", .low, ", ", .high, 
        ")"))
}

#'@export
rxUdfUi.logit <- rxUdfUi.rxpois

## nocov end
# This file is generated by .generateRandomUiFuns() in build.R
## nocov start
.expit <- function(x, low = 0, high = 1) {
    .x <- as.character(substitute(x))
    .dp <- deparse1(substitute(x))
    .tmp <- suppressWarnings(try(force(x), silent = TRUE))
    .x <- .uiArg(.x, .tmp, .dp)
    .low <- as.character(substitute(low))
    .dp <- deparse1(substitute(low))
    .tmp <- suppressWarnings(try(force(low), silent = TRUE))
    .low <- .uiArg(.low, .tmp, .dp)
    .high <- as.character(substitute(high))
    .dp <- deparse1(substitute(high))
    .tmp <- suppressWarnings(try(force(high), silent = TRUE))
    .high <- .uiArg(.high, .tmp, .dp)
    list(replace = paste0("expit(", .x, ", ", .low, ", ", .high, 
        ")"))
}

#'@export
rxUdfUi.expit <- rxUdfUi.rxpois

## nocov end
