## created by .createRxUiBlessedList() in ui-assign-parts.R edit there
.rxUiBlessed <- c("covLhsDf", "covariates", "curHi", "curLow", 
    "errParams0", "estNotAllowed", "eta", "etaLhsDf", "extraCmt", 
    "extraDvid", "hasErrors", "iniDf", "level", "levelLhsDf", 
    "lstExpr", "meta", "mixProbs", "model", "modelName", "mu2RefCovariateReplaceDataFrame", 
    "muRefCovariateDataFrame", "muRefCovariateEmpty", "muRefCurEval", 
    "muRefDataFrame", "muRefDropParameters", "muRefExtra", "muRefExtraEmpty", 
    "mv0", "mvL", "nonMuEtas", "oneTheta", "predDf", "redo", 
    "singleTheta", "sticky", "thetaLhsDf", "uiUseData", "uiUseMv")
