% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rudf.R
\name{.symengineFs}
\alias{.symengineFs}
\title{Return symengineFs from user functions}
\usage{
.symengineFs()
}
\value{
symengineFs from user functions
}
\description{
Return symengineFs from user functions
}
\author{
Matthew L. Fidler
}
\keyword{internal}
