% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/err-foceiBase.R
\name{.rxGetPredictionDVTransform}
\alias{.rxGetPredictionDVTransform}
\title{Get the Observation transformation}
\usage{
.rxGetPredictionDVTransform(env, pred1, yj)
}
\arguments{
\item{env}{Environment for the parsed model}

\item{pred1}{The \code{data.frame} of the current error}

\item{yj}{The transformation number for the current error}
}
\value{
The transformation expression
}
\description{
Get the Observation transformation
}
\author{
Matthew Fidler
}
\keyword{internal}
