% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/indLin.R
\name{rxSensMatExp}
\alias{rxSensMatExp}
\title{Differentiate and expand a matrix exponential model with forward sensitivities}
\usage{
rxSensMatExp(
  model,
  calcSens,
  calcSens2 = NULL,
  calcSens3 = NULL,
  doConst = FALSE,
  env = NULL
)
}
\arguments{
\item{model}{rxode2 model, text, or function}

\item{calcSens}{A character vector of parameter names for which sensitivities should be calculated.}

\item{calcSens2}{character vector (or \code{NULL}) requesting second-order
sensitivities \verb{rx__sens_<x>_BY_<p>_BY_<q>__} (\code{p} over \code{calcSens}, \code{q} over
\code{calcSens2}; every \code{calcSens2} element must also be in \code{calcSens}).
Expressed as \code{k_from_to} micro-constant transfers like the first-order
ones.  Ignored for \code{linCmt()} states (those use Stan forward-AD).}

\item{calcSens3}{character vector (or \code{NULL}) requesting third-order
sensitivities \verb{rx__sens_<x>_BY_<p>_BY_<q>_BY_<r>__} (\code{r} over \code{calcSens3}).
Requires \code{calcSens2}; every \code{calcSens3} element must also be in \code{calcSens2}.}

\item{doConst}{Replace constants with values; By default this is \code{FALSE}.}

\item{env}{A pre-loaded symengine environment (from \code{.rxLoadPrune()}) to
reuse instead of reloading \code{model}; when \code{NULL} it is built internally.}
}
\value{
A character string representing the matrix exponential sensitivity-expanded model code
}
\description{
Differentiate and expand a matrix exponential model with forward sensitivities
}
\author{
Matthew L. Fidler
}
