% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{rxIsCurrent}
\alias{rxIsCurrent}
\title{Checks if the rxode2 object was built with the current build}
\usage{
rxIsCurrent(obj)
}
\arguments{
\item{obj}{rxode2 family of objects}
}
\value{
boolean indicating if this was built with current rxode2
}
\description{
Checks if the rxode2 object was built with the current build
}
