% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assert.R
\name{assertVariableNew}
\alias{assertVariableNew}
\title{Assert a variable would be new to the model}
\usage{
assertVariableNew(ui, x)
}
\arguments{
\item{ui}{rxode2 ui model}

\item{x}{would the variable \code{x} variable be new in the model}
}
\value{
nothing, but will error if \code{x} would not be new
}
\description{
Assert a variable would be new to the model
}
\seealso{
Other Assertions:
\code{\link[=assertCompartmentExists]{assertCompartmentExists()}},
\code{\link[=assertCompartmentName]{assertCompartmentName()}},
\code{\link[=assertCompartmentNew]{assertCompartmentNew()}},
\code{\link[=assertRxUi]{assertRxUi()}},
\code{\link[=assertVariableExists]{assertVariableExists()}},
\code{\link[=testIniDf]{testIniDf()}},
\code{\link[=testRxUnbounded]{testRxUnbounded()}}
}
\author{
Matthew L. Fidler
}
\concept{Assertions}
