% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxIndLin.R
\name{rxIndLinStrategy}
\alias{rxIndLinStrategy}
\title{This sets the inductive linearization strategy for matrix building}
\usage{
rxIndLinStrategy(strategy = c("curState", "split"))
}
\arguments{
\item{strategy}{The strategy for inductive linearization matrix building
\itemize{
\item \code{curState} Prefer parameterizing in terms of the current
state, followed by the first state observed in the term.
\item \code{split} Split the parameterization between all states in the
term by dividing each by the number of states in the term and then
adding a matrix term for each state.
}}
}
\value{
Nothing
}
\description{
When there is more than one state in a ODE that cannot be
separated this specifies how it is incorporated into the matrix
exponential.
}
\author{
Matthew L. Fidler
}
