% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/evidPush.R, R/rudfui.R, R/rxrandom.R
\name{rxUdfUi.evid_}
\alias{rxUdfUi.evid_}
\alias{rxUdfUi.bolus}
\alias{rxUdfUi.infuse}
\alias{rxUdfUi.infuseDur}
\alias{rxUdfUi.replace}
\alias{rxUdfUi.multiply}
\alias{rxUdfUi.phantom}
\alias{rxUdfUi.obs}
\alias{rxUdfUi}
\alias{rxUdfUi.rinbinom}
\title{This function is called when processing rxode2 user functions from
the models}
\usage{
\method{rxUdfUi}{evid_}(fun)

\method{rxUdfUi}{bolus}(fun)

\method{rxUdfUi}{infuse}(fun)

\method{rxUdfUi}{infuseDur}(fun)

\method{rxUdfUi}{replace}(fun)

\method{rxUdfUi}{multiply}(fun)

\method{rxUdfUi}{phantom}(fun)

\method{rxUdfUi}{obs}(fun)

rxUdfUi(fun)

\method{rxUdfUi}{rinbinom}(fun)
}
\arguments{
\item{fun}{this is the function that needs to be parsed and
changed.  This is a R language expression}
}
\value{
This needs to return a list with the following elements:
\itemize{
\item \code{iniDf} -- the modified initial estimate data.frame
\item \code{before} -- any model code that needs to be added before the current line
\item \code{after} -- any model code that needs to be added after the current line
\item \code{replace} -- replacement code for this user function
}
}
\description{
This function is called when processing rxode2 user functions from
the models
}
\author{
Matthew L. Fidler
}
\keyword{internal}
