% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tran.R
\name{rxode2parseAssignTranslation}
\alias{rxode2parseAssignTranslation}
\title{This assigns the c level linkages for a roxde2 model}
\usage{
rxode2parseAssignTranslation(df)
}
\arguments{
\item{df}{data frame containing the character column names rxFun,
fun, type, package, packageFun and the integer column names
argMin and argMax}
}
\value{
Nothing called for side effects
}
\description{
This assigns the c level linkages for a roxde2 model
}
\examples{

rxode2parseAssignTranslation(rxode2parseGetTranslation())

}
\author{
Matthew L. Fidler
}
