% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxrandom.R
\name{rxGetSeed}
\alias{rxGetSeed}
\title{Get the rxode2 seed}
\usage{
rxGetSeed()
}
\value{
rxode2 seed state or -1 when the seed isn't set
}
\description{
Get the rxode2 seed
}
\examples{

# without setting seed

rxGetSeed()
# Now set the seed
rxSetSeed(42)

rxGetSeed()

rxnorm()

rxGetSeed()

# don't use the rxode2 seed again

rxSetSeed(-1)

rxGetSeed()

rxnorm()

rxGetSeed()

}
\seealso{
rxSetSeed, rxWithSeed, rxWithPreserveSeed
}
