% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getBaseSim.R
\name{getBaseIniSimModel}
\alias{getBaseIniSimModel}
\title{Get the base simulation model for simulation with inis in the
underlying rxode2 model}
\usage{
getBaseIniSimModel(obj)
}
\arguments{
\item{obj}{Fit Object}
}
\value{
Simulation object
}
\description{
Get the base simulation model for simulation with inis in the
underlying rxode2 model
}
\author{
Matthew L. Fidler
}
\keyword{internal}
