% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxRaw.R
\name{rxSerialize}
\alias{rxSerialize}
\title{Serialize an R Object to a Raw Vector}
\usage{
rxSerialize(x, type = c("xz", "bzip2", "base"))
}
\arguments{
\item{x}{object to serialize}

\item{type}{serialization type; one of "base", "xz" or "bzip2".}
}
\value{
raw vector
}
\description{
Serialize an R Object to a Raw Vector
}
\examples{

rxSerialize(mtcars)

rxRawToC(mtcars)

}
\author{
Matthew L. Fidler
}
\keyword{internal}
