% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/piping.R
\name{.copyUi}
\alias{.copyUi}
\title{This copies the rxode2 UI object so it can be modified}
\usage{
.copyUi(ui)
}
\arguments{
\item{ui}{Original UI object}
}
\value{
Copied UI object
}
\description{
This copies the rxode2 UI object so it can be modified
}
\author{
Matthew L. Fidler
}
