% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{setCvodeLinearSolver}
\alias{setCvodeLinearSolver}
\title{Set the CVODE linear solver}
\usage{
setCvodeLinearSolver(type)
}
\arguments{
\item{type}{Integer: 1=dense (default), 2=band (currently aliases to dense),
3=gmres, 4=bicgstab, 5=tfqmr}
}
\value{
NULL (invisibly)
}
\description{
Set the CVODE linear solver
}
\keyword{internal}
