% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assert.R
\name{assertCompartmentNew}
\alias{assertCompartmentNew}
\title{Verify that a compartment would be new to the model}
\usage{
assertCompartmentNew(ui, x)
}
\arguments{
\item{ui}{is the model to test that a model paramet exists}

\item{x}{The value to test}
}
\value{
The value or an error
}
\description{
Verify that a compartment would be new to the model
}
\seealso{
Other Assertions:
\code{\link[=assertCompartmentExists]{assertCompartmentExists()}},
\code{\link[=assertCompartmentName]{assertCompartmentName()}},
\code{\link[=assertRxUi]{assertRxUi()}},
\code{\link[=assertVariableExists]{assertVariableExists()}},
\code{\link[=assertVariableNew]{assertVariableNew()}},
\code{\link[=testIniDf]{testIniDf()}},
\code{\link[=testRxUnbounded]{testRxUnbounded()}}
}
\author{
Matthew Fidler & Bill Denney
}
\concept{Assertions}
