% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R, R/rxode2.R
\name{rxDynUnload}
\alias{rxDynUnload}
\alias{rxUnload}
\title{Unload rxode2 object}
\usage{
rxDynUnload(obj)

rxUnload(obj)
}
\arguments{
\item{obj}{A rxode2 family of objects}
}
\value{
Boolean returning if the rxode2 library is loaded.
}
\description{
Unload rxode2 object
}
\author{
Matthew L.Fidler
}
\keyword{internal}
