% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/etNew.R
\name{is.rxEt}
\alias{is.rxEt}
\title{Check if object is an rxEt event table}
\usage{
is.rxEt(x)
}
\arguments{
\item{x}{object to test}
}
\value{
logical
}
\description{
Check if object is an rxEt event table
}
