% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxValidate.R
\name{rxValidate}
\alias{rxValidate}
\alias{rxTest}
\title{Validate rxode2
This allows easy validation/qualification of nlmixr by running the
testing suite on your system.}
\usage{
rxValidate(type = NULL, skipOnCran = TRUE)

rxTest(type = NULL, skipOnCran = TRUE)
}
\arguments{
\item{type}{Type of test or filter of test type, When this is an
expression, evaluate the contents, respecting \code{skipOnCran}.  In
the expression form, stray progress messages are muffled unless
\code{options(rxode2.test.verbose = TRUE)} is set.}

\item{skipOnCran}{when \code{TRUE} skip the test on CRAN.}
}
\value{
nothing
}
\description{
Validate rxode2
This allows easy validation/qualification of nlmixr by running the
testing suite on your system.
}
\author{
Matthew L. Fidler
}
