% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{rxSeedEng}
\alias{rxSeedEng}
\title{This seeds the engine based on the number of cores used in random number generation}
\usage{
rxSeedEng(ncores = 1L)
}
\arguments{
\item{ncores}{is the number of cores to use.}
}
\value{
Nothing, called for side effects
}
\description{
This seeds the engine based on the number of cores used in random number generation
}
\examples{
rxSeedEng()
}
\keyword{internal}
