% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R, R/rxode2.R
\name{rxDynLoad}
\alias{rxDynLoad}
\alias{rxLoad}
\title{Load rxode2 object}
\usage{
rxDynLoad(obj)

rxLoad(obj)
}
\arguments{
\item{obj}{A rxode2 family of objects}
}
\value{
Boolean returning if the rxode2 library is loaded.
}
\description{
Load rxode2 object
}
\author{
Matthew L.Fidler
}
\keyword{internal}
