% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/et.R
\name{rxEtDispatchSolve}
\alias{rxEtDispatchSolve}
\alias{rxEtDispatchSolve.default}
\title{Dispatch solve to 'rxode2' solve}
\usage{
rxEtDispatchSolve(x, ...)

\method{rxEtDispatchSolve}{default}(x, ...)
}
\arguments{
\item{x}{rxode2 solve dispatch object}

\item{...}{other arguments}
}
\value{
if 'rxode2'  is loaded, a solved object, otherwise an error
}
\description{
Dispatch solve to 'rxode2' solve
}
\author{
Matthew L. Fidler
}
