% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{rxStateOde}
\alias{rxStateOde}
\title{Get the ODE states only}
\usage{
rxStateOde(obj)
}
\arguments{
\item{obj}{rxode2 object}
}
\value{
ODE states only
}
\description{
Get the ODE states only
}
\examples{
\donttest{
mod <- rxode2({
  Cp <- linCmt(Cl, V, Q2, V2, Q3, V3)
  ke0 <- log(2)/(50)
  d/dt(Ce) <- (Cp-Ce)*ke0
})

rxStateOde(mod)

rxState(mod)

mod <- rxode2({
  Cp <- linCmt(Cl, V, Q2, V2, Q3, V3, ka)
  ke0 <- log(2)/(50)
  d/dt(Ce) <- (Cp-Ce)*ke0
}, linCmtSens="linCmtB")

rxStateOde(mod)

rxState(mod)
}

}
\author{
Matthew L. Fidler
}
