% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/evidPush.R
\name{obs}
\alias{obs}
\title{Add observations to a model}
\usage{
obs(...)
}
\arguments{
\item{...}{Numeric values to be added as observations after the
current time \code{t}.  They would be added as events with \code{evid = 0}
and \code{amt = 0} at the specified future time points.  The time
points are relative to the current model time \code{t} (i.e.,
\code{obs(12)} adds an observation at \code{t + 12}).  If this is a ui
model, you can also specify observations with obs(seq(0, 24,
by=0.5)) to add observations every 0.5 time units from t to t+24
since it will resolve to the obs() inside of the final rxode2
model.}
}
\value{
This function is only meaningful inside an rxode2 model; it
returns \code{NULL} invisibly if called from R directly (after signaling
an error).
}
\description{
Add observations to a model
}
\details{
\subsection{Behavior inside a model}{

\code{obs()} is evaluated at every output time point (when the solver is
exactly at a scheduled event time).  The pushed event is inserted into
the individual's event timeline and the solver visits it now or at the
specified future time.

The number of events that may be pushed per individual is limited by
the \code{maxExtra} argument of \code{\link[=rxSolve]{rxSolve()}}.  When \code{maxExtra = 0}
(the default) there is no limit.  Exceeding the limit causes an
error.

Past-time pushes (where \code{time < t}) are silently ignored and counted;
a warning is issued after solving.
}
}
\author{
Matthew L. Fidler
}
