% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tran.R
\name{rxode2parseGetPointerAssignment}
\alias{rxode2parseGetPointerAssignment}
\title{This function gets the currently assigned function pointer assignments}
\usage{
rxode2parseGetPointerAssignment()
}
\value{
The currently assigned pointer assignments
}
\description{
This function gets the currently assigned function pointer assignments
}
\examples{
rxode2parseGetTranslation()
}
\author{
Matthew L. Fidler
}
