% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/d.R
\name{rxode2parseD}
\alias{rxode2parseD}
\title{This gives the derivative table for rxode2}
\usage{
rxode2parseD()
}
\value{
Derivative table environment for rxode2
}
\description{
This will help allow registration of functions in \code{rxode2}
}
\details{
This environment is a derivative table;

For example:

Derivative(f(a,b,c), a) = fa()
Derivative(f(a,b,c), b) = fb()
Derivative(f(a,b,c), c) = fc()

Then the derivative table for \code{f} would be:

assign("f", list(fa(a,b,c), fb(a,b,c), fc(a,b,c)), rxode2parseD())

fa translates the arguments to the derivative with respect to a
fb translates the arguments to the derivative with respect to b

If any of the list is NULL then rxode2 won't know how to take a
derivative with respect to the argument.

If the list is shorter than the length of the arguments then the
argument then the derivative of arguments that are not specified
cannot be taken.
}
\author{
Matthew L. Fidler
}
