% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/err-foceiBase.R
\name{.handleSingleErrTypeNormOrTFoceiBase}
\alias{.handleSingleErrTypeNormOrTFoceiBase}
\title{Handle the single error for normal or t distributions}
\usage{
.handleSingleErrTypeNormOrTFoceiBase(
  env,
  pred1,
  errNum = 1L,
  rxPredLlik = TRUE,
  arNorm = FALSE
)
}
\arguments{
\item{env}{Environment for the parsed model}

\item{pred1}{The \code{data.frame} of the current error}

\item{errNum}{The number of the error specification in the nlmixr2 model}

\item{rxPredLlik}{A boolean indicating if the log likelihood should
be calculated for non-normal distributions.  By default \code{TRUE}.}

\item{arNorm}{A boolean; when \code{TRUE}, emit the AR(1) norm
(conditional mean/variance) form used by the focei inner model.
By default \code{FALSE}.}
}
\value{
A list of the lines added.  The lines will contain
\itemize{
\item \code{rx_yj_} which is an integer that corresponds to the
transformation type.
\item \code{rx_lambda_} is the transformation lambda
\item \code{rx_low_} The lower boundary of the transformation
\item \code{rx_hi_} The upper boundary of the transformation
\item \code{rx_pred_f_} The prediction function
\item \code{rx_pred_} The transformed prediction function
\item \code{rx_r_} The transformed variance
}
}
\description{
Handle the single error for normal or t distributions
}
\author{
Matthew Fidler
}
