% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxPrune.R
\name{rxPrune}
\alias{rxPrune}
\title{Prune branches (ie if/else) from rxode2}
\usage{
rxPrune(x)
}
\arguments{
\item{x}{rxode2 model that can be accessed by rxNorm}
}
\value{
Pruned rxode2 model text.
}
\description{
This prunes branches (ie if/else) from rxode2.
}
\author{
Matthew L. Fidler
}
\keyword{internal}
