% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxStack.R
\name{is.rxStackData}
\alias{is.rxStackData}
\title{Return if the object can be stacked}
\usage{
is.rxStackData(object)
}
\arguments{
\item{object}{object to test if it can be stacked}
}
\value{
boolean to tell if an object can be stacked using rxode2
}
\description{
Return if the object can be stacked
}
\examples{

is.rxStackData(NULL)
}
\author{
Matthew L. Fidler
}
