% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxrandom.R
\name{rxord}
\alias{rxord}
\title{Simulate ordinal value}
\usage{
rxord(...)
}
\arguments{
\item{...}{the probabilities to be simulated.  These should sum up to a number below one.}
}
\value{
A number from 1 to the (number of probabilities specified + 1)
}
\description{
Simulate ordinal value
}
\details{
The values entered into the 'rxord' simulation will simulate the
probability of falling each group. If it falls outside of the
specified probabilities, it will simulate the group (number of
probabilities specified + 1)
}
\examples{

# This will give values 1, and 2
rxord(0.5)
rxord(0.5)
rxord(0.5)
rxord(0.5)

# This will give values 1, 2 and 3
rxord(0.3, 0.3)
rxord(0.3, 0.3)
rxord(0.3, 0.3)

}
\author{
Matthew L. Fidler
}
