% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ui-getControl.R
\name{rxGetControl}
\alias{rxGetControl}
\title{rxGetControl option from ui}
\usage{
rxGetControl(ui, option, default)
}
\arguments{
\item{ui}{rxode2 ui object}

\item{option}{Option to get}

\item{default}{Default value}
}
\value{
Option (if present) or default value
}
\description{
rxGetControl option from ui
}
\author{
Matthew L. Fidler
}
