% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxRaw.R
\name{rxDeserialize}
\alias{rxDeserialize}
\title{Deserialize a Raw Vector or String to an R Object}
\usage{
rxDeserialize(x)
}
\arguments{
\item{x}{raw vector or string to deserialize}
}
\value{
R object
}
\description{
Deserialize a Raw Vector or String to an R Object
}
\examples{

rxDeserialize(rxSerialize(mtcars))

}
\author{
Matthew L. Fidler
}
\keyword{internal}
