% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.rxLinCmt}
\alias{.rxLinCmt}
\title{Get the linear compartment model states}
\usage{
.rxLinCmt(obj)
}
\arguments{
\item{obj}{rxode2 type of object}
}
\value{
compartment names
}
\description{
Get the linear compartment model states
}
\author{
Matthew L. Fidler
}
\keyword{internal}
