% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{rxCat}
\alias{rxCat}
\title{Use cat when rxode2.verbose is TRUE}
\usage{
rxCat(a, ...)
}
\arguments{
\item{...}{Parameters sent to cat}
}
\value{
nothing
}
\description{
Use cat when rxode2.verbose is TRUE
}
\author{
Matthew L. Fidler
}
\keyword{internal}
