% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxode-options.R
\name{.rxode2ptrs}
\alias{.rxode2ptrs}
\title{Get the rxode2 function pointers}
\usage{
.rxode2ptrs()
}
\value{
a list of function pointers
}
\description{
This function is used to get the function pointers for rxode2.  This is
used to allow rxode2 to have binary linkage to nlmixr2est.
}
\examples{

.rxode2ptrs()
}
\author{
Matthew L. Fidler
}
