% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ui-assign-parts.R
\name{model<-}
\alias{model<-}
\title{Assign the model block in the rxode2 related object}
\usage{
model(x, envir = environment(x)) <- value
}
\arguments{
\item{x}{rxode2 related object}

\item{envir}{Environment where assignment occurs}

\item{value}{Value of the object}
}
\value{
rxode2 related object
}
\description{
Assign the model block in the rxode2 related object
}
\author{
Matthew L. Fidler
}
