% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tran.R
\name{rxode2parseGetPackagesToLoad}
\alias{rxode2parseGetPackagesToLoad}
\alias{rxode2parseAssignPackagesToLoad}
\title{Control the packages that are loaded when a \code{rxode2} model dll is loaded}
\usage{
rxode2parseGetPackagesToLoad()

rxode2parseAssignPackagesToLoad(pkgs = rxode2parseGetPackagesToLoad())
}
\arguments{
\item{pkgs}{The packages to make sure are loaded every time you load an rxode2 model.}
}
\value{
List of packages to load
}
\description{
Control the packages that are loaded when a \code{rxode2} model dll is loaded
}
\examples{

rxode2parseGetPackagesToLoad()

rxode2parseAssignPackagesToLoad(rxode2parseGetPackagesToLoad())
}
\author{
Matthew Fidler
}
