% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxJacobian.R
\name{.rxSens}
\alias{.rxSens}
\title{Sensitivity for model}
\usage{
.rxSens(model, vars, vars2, vars3)
}
\arguments{
\item{model}{symengine model environment}

\item{vars}{Variables for single sensitivity}

\item{vars2}{if present, 2 parameter sensitivity}
}
\value{
Sensitivity
}
\description{
Sensitivity for model
}
\author{
Matthew L. Fidler
}
\keyword{internal}
