% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxRaw.R
\name{rxRawToC}
\alias{rxRawToC}
\title{Convert a Raw Vector or R object to C Code}
\usage{
rxRawToC(raw, type = c("xz", "base", "bzip2"))
}
\arguments{
\item{raw}{vector or R object to convert}

\item{type}{serialization type; one of "base", "xz" or "bzip2".}
}
\value{
raw vector for conversion to C in the codegen procedure
}
\description{
This function converts a raw vector or R object to C code that
is used in the rxode2 model
}
\examples{

rxRawToC(mtcars)

}
\author{
Matthew L. Fidler
}
\keyword{internal}
