% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxode-options.R
\name{rxTempDir}
\alias{rxTempDir}
\title{Get the rxode2 temporary directory}
\usage{
rxTempDir()
}
\value{
rxode2 temporary directory.
}
\description{
Get the rxode2 temporary directory
}
