% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrelu.R
\name{dlReLU}
\alias{dlReLU}
\title{Derivative of Leaky ReLU activation function}
\usage{
dlReLU(x)
}
\arguments{
\item{x}{numeric vector}
}
\value{
numeric vector
}
\description{
Derivative of Leaky ReLU activation function
}
\examples{

dlReLU(c(-1, 0, 1))

\donttest{
# Can use in rxode2 as well

r <- rxode2({r <- dlReLU(time)})
e <- et(c(-1, 0, 1))
rxSolve(r, e)
}

}
\seealso{
Other Activation Functions:
\code{\link[=ELU]{ELU()}},
\code{\link[=GELU]{GELU()}},
\code{\link[=PReLU]{PReLU()}},
\code{\link[=ReLU]{ReLU()}},
\code{\link[=SELU]{SELU()}},
\code{\link[=Swish]{Swish()}},
\code{\link[=dELU]{dELU()}},
\code{\link[=dGELU]{dGELU()}},
\code{\link[=dPReLU]{dPReLU()}},
\code{\link[=dReLU]{dReLU()}},
\code{\link[=dSELU]{dSELU()}},
\code{\link[=dSwish]{dSwish()}},
\code{\link[=dsoftplus]{dsoftplus()}},
\code{\link[=lReLU]{lReLU()}},
\code{\link[=softplus]{softplus()}}
}
\concept{Activation Functions}
