% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rudfui.R
\name{rxUdfUiNum}
\alias{rxUdfUiNum}
\title{This gives the current number in the ui of the particular function being called.}
\usage{
rxUdfUiNum()
}
\value{
integer greater than 1L
}
\description{
If this is called outside of function parsing or the input is
unexpected this returns 1L. This is useful when writing replacement
UI functions
}
\examples{

rxUdfUiNum()
}
\seealso{
Other User functions:
\code{\link[=linMod]{linMod()}},
\code{\link[=rxUdfUiControl]{rxUdfUiControl()}},
\code{\link[=rxUdfUiData]{rxUdfUiData()}},
\code{\link[=rxUdfUiEst]{rxUdfUiEst()}},
\code{\link[=rxUdfUiIniLhs]{rxUdfUiIniLhs()}},
\code{\link[=rxUdfUiMv]{rxUdfUiMv()}},
\code{\link[=rxUdfUiParsing]{rxUdfUiParsing()}}
}
\author{
Matthew L. Fidler
}
\concept{User functions}
