% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxJacobian.R
\name{.rxJacobian}
\alias{.rxJacobian}
\title{Internal function for calculating the Jacobian}
\usage{
.rxJacobian(model, vars = TRUE)
}
\arguments{
\item{model}{symengine environment}

\item{vars}{Variables}
}
\value{
Jacobian information
}
\description{
Internal function for calculating the Jacobian
}
\author{
Matthew L. Fidler
}
\keyword{internal}
