% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxRaw.R
\name{rxOldQsDes}
\alias{rxOldQsDes}
\title{Deserialize a Raw Vector using qs}
\usage{
rxOldQsDes(x)
}
\arguments{
\item{x}{raw vector to deserialize using qs (not on CRAN any more)}
}
\value{
qs::deserialize (if qs is available) of x
}
\description{
Deserialize a Raw Vector using qs
}
\author{
Matthew L. Fidler
}
\keyword{internal}
