% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adjointDiscrete.R
\name{.rxAdjointMethodStiff}
\alias{.rxAdjointMethodStiff}
\title{Does an adjoint (\code{s}) solve method require the stiff analytic Jacobian?}
\usage{
.rxAdjointMethodStiff(method)
}
\arguments{
\item{method}{adjoint method name(s) -- a single name, a composite vector
\code{c("dop853s","ros4s")}, or a \code{"dop853s+ros4s"} string.}
}
\value{
\code{TRUE} if any component is a stiff adjoint method.
}
\description{
Does an adjoint (\code{s}) solve method require the stiff analytic Jacobian?
}
\author{
Matthew L. Fidler
}
\keyword{internal}
