% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{rxExpandSens_}
\alias{rxExpandSens_}
\title{Expand sensitivity}
\usage{
rxExpandSens_(state, calcSens)
}
\arguments{
\item{state}{is the state to expand}

\item{calcSens}{is the variables to calculate sensitivity}
}
\value{
symengine string for expanded sensitivity
}
\description{
Expand sensitivity
}
\keyword{internal}
