% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxOom.R
\name{rxEventTableFile}
\alias{rxEventTableFile}
\title{Create a file-backed event table reference}
\usage{
rxEventTableFile(
  path,
  format = c("auto", "parquet", "csv", "fst", "rds"),
  id = "id"
)
}
\arguments{
\item{path}{path to the file containing the event table}

\item{format}{file format: "auto" (detect from extension), "parquet", "csv", "fst", or "rds"}

\item{id}{name of the subject ID column}
}
\value{
An \code{rxEtFile} object
}
\description{
Create a file-backed event table reference
}
