% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxode2.R
\name{rxLastCompile}
\alias{rxLastCompile}
\title{Get the last compiled model information as alist}
\usage{
rxLastCompile()
}
\value{
A list contains the following elements:
\itemize{
\item \code{msg} the message for a bad compilation, or NULL if successful.
\item \code{stdout} the standard output from the compilation
\item \code{stderr} the standard error from the compilation
\item \code{c} the code code that was used
}

This list will be returned invisibly, but the function will
also message the contents to the console.
}
\description{
Get the last compiled model information as alist
}
\examples{
\donttest{
rxode2({
  a <- b
})
rxLastCompile()
}
}
\author{
Matthew L. Fidler
}
