% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rudf.R
\name{.rxSEeqUsr}
\alias{.rxSEeqUsr}
\title{Return the equivalents symengine user functions from C}
\usage{
.rxSEeqUsr()
}
\value{
equivalent symengine user functions
}
\description{
Return the equivalents symengine user functions from C
}
\author{
Matthew L. Fidler
}
\keyword{internal}
