% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxrandom.R
\name{.rxSetSeed}
\alias{.rxSetSeed}
\title{Set the random seed for the session}
\usage{
.rxSetSeed(seed)
}
\arguments{
\item{seed}{Seed from \code{.rxGetSeed()}}
}
\value{
Nothing, called for its side effects
}
\description{
This sets the random seed for the session.  This is used internally
to set the random seed for the session.
}
\author{
Matthew L. Fidler
}
\keyword{internal}
