% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{summary.rxode2}
\alias{summary.rxode2}
\title{Print expanded information about the rxode2 object.}
\usage{
\method{summary}{rxode2}(object, ...)
}
\arguments{
\item{object}{rxode2 object}

\item{...}{Ignored parameters}
}
\value{
object is returned
}
\description{
This prints the expanded information about the rxode2 object.
}
\author{
Matthew L.Fidler
}
