% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxrandom.R
\name{.cbindOme}
\alias{.cbindOme}
\title{cbind Ome}
\usage{
.cbindOme(et, mat, n)
}
\arguments{
\item{et}{The theta data frame}

\item{mat}{The full matrix simulation from omegas}

\item{n}{number of subject simulated}
}
\value{
data frame with et combined with simulated omega matrix values
}
\description{
cbind Ome
}
\author{
Matthew Fidler
}
