% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/et.R
\name{.pipeRx}
\alias{.pipeRx}
\alias{.pipeInits}
\alias{.pipeEvents}
\alias{.pipeParams}
\alias{.pipeKeep}
\alias{.pipeThetaMat}
\alias{.pipeOmega}
\alias{.pipeSigma}
\alias{.pipeDfObs}
\alias{.pipeDfSub}
\alias{.pipeNSub}
\alias{.pipeNStud}
\title{Assign in the rxode2 pipeline}
\usage{
.pipeRx(obj)

.pipeInits(obj)

.pipeEvents(obj)

.pipeParams(obj)

.pipeKeep(obj)

.pipeThetaMat(obj)

.pipeOmega(obj)

.pipeSigma(obj)

.pipeDfObs(obj)

.pipeDfSub(obj)

.pipeNSub(obj)

.pipeNStud(obj)
}
\arguments{
\item{obj}{Object to assign.  If NA return the value.}
}
\value{
The pipeline object (invisibly)
}
\description{
Assign in the rxode2 pipeline
}
\author{
Matthew L. Fidler
}
\keyword{internal}
