% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{rxSetupScale}
\alias{rxSetupScale}
\title{Setup the initial conditions.}
\usage{
rxSetupScale(obj, scale = NULL, extraArgs = NULL)
}
\arguments{
\item{obj}{rxode2 object}

\item{scale}{A numeric vector scales}

\item{extraArgs}{A list of extra args to parse for initial conditions.}
}
\value{
setup scale for changing compartment values
}
\description{
Setup the initial conditions.
}
\author{
Matthew L. Fidler
}
\keyword{internal}
