% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rudfui.R
\name{rxUdfUiExpr}
\alias{rxUdfUiExpr}
\title{Give the expression as a compressed model or expression}
\usage{
rxUdfUiExpr(expr, env = parent.frame())
}
\arguments{
\item{expr}{the R language expression to evaluate}

\item{env}{the environment in which to evaluate the expression}
}
\value{
expression
}
\description{
Here it means that it evaluates to a variable or a number.
}
\author{
Matthew L. Fidler
}
