% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{rxGetSerialType_}
\alias{rxGetSerialType_}
\title{Get the serialization type from raw vector}
\usage{
rxGetSerialType_(raw)
}
\arguments{
\item{raw}{A raw vector}
}
\value{
a string indicating the serialization type:
"qs2", "qdata", "qs", "bzip2", "xz",  "base", or "unknown"
}
\description{
Get the serialization type from raw vector
}
\keyword{internal}
