% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{erf}
\alias{erf}
\title{Error function}
\usage{
erf(x)
}
\arguments{
\item{x}{vector or real values}
}
\value{
erf of x
}
\description{
Error function
}
\examples{
erf(1.0)
}
\author{
Matthew L. Fidler
}
