% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ui-getControl.R
\name{rxRemoveControl}
\alias{rxRemoveControl}
\title{rxRemoveControl options for UI object}
\usage{
rxRemoveControl(ui)
}
\arguments{
\item{ui}{rxode2 ui object}
}
\value{
Nothing, called for side effects
}
\description{
rxRemoveControl options for UI object
}
\author{
Matthew L. Fidler
}
