% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxode2.R
\name{rxReload}
\alias{rxReload}
\title{Reload rxode2 DLL}
\usage{
rxReload()
}
\value{
boolean of if the object is reloaded
}
\description{
Can be useful for debugging
}
\author{
Matthew L. Fidler
}
\keyword{internal}
