% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gelu.R
\name{dGELU}
\alias{dGELU}
\alias{d2GELU}
\alias{d3GELU}
\alias{d4GELU}
\title{Derivatives of GELU}
\usage{
dGELU(x)

d2GELU(x)

d3GELU(x)

d4GELU(x)
}
\arguments{
\item{x}{numeric vector}
}
\value{
numeric vector
}
\description{
Derivatives of GELU
}
\examples{
dGELU(c(-2, -1, 0, 1, 2))
d2GELU(c(-2, -1, 0, 1, 2))
d3GELU(c(-2, -1, 0, 1, 2))
d4GELU(c(-2, -1, 0, 1, 2))
\donttest{
# you can use rxode2 as well
r <- rxode2({
   r1 <- dGELU(time)
   r2 <- d2GELU(time)
   r3 <- d3GELU(time)
   r4 <- d4GELU(time)
})
et <- et(c(-2, -1, 0, 1, 2))
rxSolve(r, et)
}
}
\seealso{
Other Activation Functions:
\code{\link[=ELU]{ELU()}},
\code{\link[=GELU]{GELU()}},
\code{\link[=PReLU]{PReLU()}},
\code{\link[=ReLU]{ReLU()}},
\code{\link[=SELU]{SELU()}},
\code{\link[=Swish]{Swish()}},
\code{\link[=dELU]{dELU()}},
\code{\link[=dPReLU]{dPReLU()}},
\code{\link[=dReLU]{dReLU()}},
\code{\link[=dSELU]{dSELU()}},
\code{\link[=dSwish]{dSwish()}},
\code{\link[=dlReLU]{dlReLU()}},
\code{\link[=dsoftplus]{dsoftplus()}},
\code{\link[=lReLU]{lReLU()}},
\code{\link[=softplus]{softplus()}}
}
\concept{Activation Functions}
