% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rudfui.R
\name{rxUdfUiParsing}
\alias{rxUdfUiParsing}
\title{Returns if the current ui function is being parsed}
\usage{
rxUdfUiParsing()
}
\value{
logical if the current ui function is being parsed
}
\description{
Returns if the current ui function is being parsed
}
\examples{
rxUdfUiParsing()
}
\seealso{
Other User functions:
\code{\link[=linMod]{linMod()}},
\code{\link[=rxUdfUiControl]{rxUdfUiControl()}},
\code{\link[=rxUdfUiData]{rxUdfUiData()}},
\code{\link[=rxUdfUiEst]{rxUdfUiEst()}},
\code{\link[=rxUdfUiIniLhs]{rxUdfUiIniLhs()}},
\code{\link[=rxUdfUiMv]{rxUdfUiMv()}},
\code{\link[=rxUdfUiNum]{rxUdfUiNum()}}
}
\author{
Matthew L. Fidler
}
\concept{User functions}
