% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/swish.R
\name{dSwish}
\alias{dSwish}
\title{Derivative of the Swish Activation Function}
\usage{
dSwish(x)
}
\arguments{
\item{x}{A numeric vector. All elements must be finite and
non-missing.}
}
\value{
A numeric vector where the derivative of the SELU function
has been applied to each element of \code{x}.
}
\description{
Derivative of the Swish Activation Function
}
\examples{
dSwish(c(-1, 0, 1, 2))

\donttest{
# Can also be used in rxode2:
x <- rxode2({
  r <- dSwish(time)
})
e <- et(c(-1, 0, 1, 2))
rxSolve(x, e)
}
}
\seealso{
Other Activation Functions:
\code{\link[=ELU]{ELU()}},
\code{\link[=GELU]{GELU()}},
\code{\link[=PReLU]{PReLU()}},
\code{\link[=ReLU]{ReLU()}},
\code{\link[=SELU]{SELU()}},
\code{\link[=Swish]{Swish()}},
\code{\link[=dELU]{dELU()}},
\code{\link[=dGELU]{dGELU()}},
\code{\link[=dPReLU]{dPReLU()}},
\code{\link[=dReLU]{dReLU()}},
\code{\link[=dSELU]{dSELU()}},
\code{\link[=dlReLU]{dlReLU()}},
\code{\link[=dsoftplus]{dsoftplus()}},
\code{\link[=lReLU]{lReLU()}},
\code{\link[=softplus]{softplus()}}
}
\author{
Matthew Fidler
}
\concept{Activation Functions}
