% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{rxAllowUnload}
\alias{rxAllowUnload}
\title{Allow unloading of dlls}
\usage{
rxAllowUnload(allow)
}
\arguments{
\item{allow}{boolean indicating if garbage collection will unload of rxode2 dlls.}
}
\value{
Boolean allow; called for side effects
}
\description{
Allow unloading of dlls
}
\examples{

# Garbage collection will not unload un-used rxode2 dlls
rxAllowUnload(FALSE);

# Garbage collection will unload unused rxode2 dlls
rxAllowUnload(TRUE);
}
\author{
Matthew Fidler
}
