% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxSerialize.R
\name{rxSaveState}
\alias{rxSaveState}
\title{Save the pre-integration rxode2 solver state to a binary file}
\usage{
rxSaveState(file)
}
\arguments{
\item{file}{Character(1).  Path of the file to write.}
}
\value{
Invisibly \code{TRUE}.
}
\description{
This is called automatically when \code{rxControl(serializeFile="path")} is
supplied to \code{rxSolve()}.  The file can be reloaded later by passing it
as the \code{params} argument of \code{rxSolve()} together with the matching
model object.
}
\seealso{
\code{\link{rxLoadState}}
}
\keyword{internal}
