% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getBaseSim.R
\name{getBaseSimModel}
\alias{getBaseSimModel}
\alias{getBaseSimModel.default}
\alias{getBaseIniSimModel.default}
\title{Get the base simulation model for simulation}
\usage{
getBaseSimModel(obj)

\method{getBaseSimModel}{default}(obj)

\method{getBaseIniSimModel}{default}(obj)
}
\arguments{
\item{obj}{Fit Object}
}
\value{
Simulation object
}
\description{
Get the base simulation model for simulation
}
\author{
Matthew L. Fidler
}
\keyword{internal}
