% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxode2.R
\name{.rxLastCompileSuccess}
\alias{.rxLastCompileSuccess}
\title{Was the last compilation successful?}
\usage{
.rxLastCompileSuccess(set)
}
\arguments{
\item{set}{Logical flag. If missing, returns the current compilation
status. If provided, sets the compilation status.}
}
\value{
this determines if the last compilation was successful.  This is useful for
debugging and testing purposes.
}
\description{
Was the last compilation successful?
}
\examples{

.rxLastCompileSuccess()

}
\author{
Matthew L. Fidler
}
\keyword{internal}
