% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/etTran.R
\name{.getLastIdLvl}
\alias{.getLastIdLvl}
\title{Get the last \code{idLvl}}
\usage{
.getLastIdLvl()
}
\value{
Last \code{idLvl}
}
\description{
Get the last \code{idLvl}
}
\examples{
.getLastIdLvl()
}
\author{
Matthew L. Fidler
}
\keyword{internal}
