% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxode2.R
\name{rxCondition}
\alias{rxCondition}
\title{Current Condition for rxode2 object}
\usage{
rxCondition(obj, condition = NULL)
}
\arguments{
\item{obj}{rxode2 object}

\item{condition}{If specified and is one of the conditions in the
rxode2 object (as determined by \code{\link[=rxExpandIfElse]{rxExpandIfElse()}}),
assign the rxode2 current condition to this parameter.  If the
condition is not one of the known condition, the condition is
set to \code{NULL}, implying no conditioning currently used.}
}
\value{
Current condition for rxode2 object
}
\description{
Current Condition for rxode2 object
}
\author{
Matthew L. Fidler
}
\keyword{internal}
