% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/et.R
\name{.clearPipe}
\alias{.clearPipe}
\title{Clear/Set pipeline}
\usage{
.clearPipe(
  rx = NULL,
  inits = NULL,
  events = NULL,
  params = NULL,
  iCov = NULL,
  keep = NULL,
  thetaMat = NULL,
  omega = NULL,
  sigma = NULL,
  dfObs = NULL,
  dfSub = NULL,
  nSub = NULL,
  nStud = NULL
)
}
\arguments{
\item{rx}{rxode2 object}
}
\value{
None, clears rxode2 pipeline
}
\description{
Clear/Set pipeline
}
\keyword{internal}
