% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rudfui.R
\name{rxUdfUiIniLhs}
\alias{rxUdfUiIniLhs}
\title{Return the lhs parsed language expression}
\usage{
rxUdfUiIniLhs()
}
\value{
lhs language expression or NULL
}
\description{
Return the lhs parsed language expression
}
\examples{

rxUdfUiIniLhs()

}
\seealso{
Other User functions:
\code{\link[=linMod]{linMod()}},
\code{\link[=rxUdfUiControl]{rxUdfUiControl()}},
\code{\link[=rxUdfUiData]{rxUdfUiData()}},
\code{\link[=rxUdfUiEst]{rxUdfUiEst()}},
\code{\link[=rxUdfUiMv]{rxUdfUiMv()}},
\code{\link[=rxUdfUiNum]{rxUdfUiNum()}},
\code{\link[=rxUdfUiParsing]{rxUdfUiParsing()}}
}
\author{
Matthew L. Fidler
}
\concept{User functions}
