% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{rxExpandGrid_}
\alias{rxExpandGrid_}
\title{Expand grid internal function}
\usage{
rxExpandGrid_(c1, c2, type)
}
\arguments{
\item{c1}{character vector of items to be expanded}

\item{c2}{second character vector of items to be expanded}

\item{type}{0 for a typical data frame, 1 for symengine sensitivity expansion}
}
\value{
data frame (when type = 0) or symengine string (when type=1)
}
\description{
Expand grid internal function
}
\keyword{internal}
