% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/err.R
\name{.rxBlankIni}
\alias{.rxBlankIni}
\title{Get a blank, theta1, or eta1 initialization block for iniDf}
\usage{
.rxBlankIni(type = c("empty", "theta", "eta"))
}
\arguments{
\item{type}{type of initialization block to return}
}
\value{
A data.frame with the appropriate number/type of columns.

For type="empty", the data.frame will have 0 rows but all the correct types.

For type="theta", the data.frame will have 1 row with the correct
types and default values.  The "name" and "est" will likely need to
be updated.

For type="eta", the data.frame will have 1 row with the correct
types and default values for the a single eta being added.  The
"name" and "est" will likely need to be updated.
}
\description{
Get a blank, theta1, or eta1 initialization block for iniDf
}
\examples{

.rxBlankIni("empty")

.rxBlankIni("theta")

.rxBlankIni("eta")

}
\author{
Matthew L. Fidler
}
\keyword{internal}
