% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{rxRepR0_}
\alias{rxRepR0_}
\title{Rep R0 for foce}
\usage{
rxRepR0_(neta)
}
\arguments{
\item{neta}{ETA to substitute}
}
\value{
Returns a string of R code to substitute the rx_r expression in the symengine environment .s
}
\description{
Rep R0 for foce
}
\keyword{internal}
