% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.rxode2}
\alias{print.rxode2}
\title{Print information about the rxode2 object.}
\usage{
\method{print}{rxode2}(x, ...)
}
\arguments{
\item{x}{An rxode2 object}

\item{...}{Ignored parameters}
}
\value{
original object
}
\description{
This prints the model name and its status for being able to be solved
}
\author{
Matthew L.Fidler
}
\keyword{internal}
