% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.uiArg}
\alias{.uiArg}
\title{Handle arguments for ui functions}
\usage{
.uiArg(char, f, dp)
}
\arguments{
\item{char}{This is the character equivalent of the argument}

\item{f}{This is the forced equivalent of the argument}

\item{dp}{This is deparsed expression}
}
\value{
character representing the underlying rxode2 code for the argument
}
\description{
Note this is an internal function but it is exported in case it is
useful.
}
\examples{

.uiArg("1.0", 1.0, "1.0")
}
\author{
Matthew L. Fidler
}
\keyword{internal}
