% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{rxLock}
\alias{rxLock}
\alias{rxUnlock}
\title{Lock/unlocking of rxode2 dll file}
\usage{
rxLock(obj)

rxUnlock(obj)
}
\arguments{
\item{obj}{A rxode2 family of objects}
}
\value{
nothing; called for side effects
}
\description{
Lock/unlocking of rxode2 dll file
}
