% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/err-foceiBase.R
\name{.rxGetHiBoundaryPred1AndIni}
\alias{.rxGetHiBoundaryPred1AndIni}
\title{Get the upper boundary condition when the transformation it}
\usage{
.rxGetHiBoundaryPred1AndIni(env, pred1)
}
\arguments{
\item{env}{Environment for the parsed model}

\item{pred1}{The \code{data.frame} of the current error}
}
\value{
Upper Boundary
}
\description{
Get the upper boundary condition when the transformation it
}
\author{
Matthew Fidler
}
\keyword{internal}
