% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.rxDocTable}
\alias{.rxDocTable}
\title{Print out a table in the documentation}
\usage{
.rxDocTable(table, caption = "none")
}
\arguments{
\item{table}{data frame}

\item{caption}{a character vector representing the caption for the latex table}
}
\value{
based on the \code{knitr} context:
\itemize{
\item output a \code{kableExtra::kbl} for \code{latex} output
\item output a \code{DT::datatable} for html output
\item otherwise output a \code{knitr::kable}
}
}
\description{
Print out a table in the documentation
}
\examples{
.rxDocTable(rxReservedKeywords)
}
\author{
Matthew L. Fidler
}
\keyword{internal}
