% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxrandom.R
\name{.vecDf}
\alias{.vecDf}
\title{Convert numeric vector to repeated data.frame}
\usage{
.vecDf(vec, n)
}
\arguments{
\item{vec}{Named input vector}

\item{n}{Number of columns}
}
\value{
Data frame with repeated vec
}
\description{
Convert numeric vector to repeated data.frame
}
\author{
Matthew Fidler
}
