% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{rxDfdy}
\alias{rxDfdy}
\title{Jacobian and parameter derivatives}
\usage{
rxDfdy(obj)
}
\arguments{
\item{obj}{rxode2 family of objects}
}
\value{
A list of the jacobian parameters defined in this rxode2
object.
}
\description{
Return Jacobain and parameter derivatives
}
\seealso{
Other Query model information:
\code{\link[=rxInits]{rxInits()}},
\code{\link[=rxLhs]{rxLhs()}},
\code{\link[=rxModelVars]{rxModelVars()}},
\code{\link[=rxParams]{rxParams()}},
\code{\link[=rxState]{rxState()}}
}
\author{
Matthew L. Fidler
}
\concept{Query model information}
