% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxode2.R
\name{coef.rxode2}
\alias{coef.rxode2}
\title{Return the rxode2 coefficients}
\usage{
\method{coef}{rxode2}(object, ...)
}
\arguments{
\item{object}{is an rxode2 object}

\item{...}{ignored arguments}
}
\value{
a rxCoef object with the following
\itemize{
\item \code{params}  is a list of strings for parameters for the rxode2 object
\item \code{state} is a list of strings for the names of each state in
the rxode2 object.
\item \code{ini} is the model specified default values for the
parameters.
\item \code{rxode2} is the referring rxode2 object
}
}
\description{
This returns the parameters , state variables
}
\author{
Matthew L.Fidler
}
\keyword{internal}
