% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.rxIsLinCmt}
\alias{.rxIsLinCmt}
\title{Internal function to tell if the linCmt() is the model variables}
\usage{
.rxIsLinCmt()
}
\value{
0 or 1
}
\description{
Internal function to tell if the linCmt() is the model variables
}
\author{
Matthew L. Fidler
}
\keyword{internal}
