% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxode2.R
\name{rxNumLoaded}
\alias{rxNumLoaded}
\title{Get the number of loaded rxode2 DLLs}
\usage{
rxNumLoaded()
}
\value{
Number of loaded rxode2 DLLs
}
\description{
Get the number of loaded rxode2 DLLs
}
\examples{
rxNumLoaded()
}
\author{
Matthew L. Fidler
}
