% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.getWh}
\alias{.getWh}
\title{Get the internal breakdown of an evid}
\usage{
.getWh(i)
}
\arguments{
\item{i}{evid to breakdown}
}
\value{
named evid integer vector
}
\description{
Get the internal breakdown of an evid
}
\examples{

.getWh(1001)
.getWh(10401)

}
\author{
Matthew L. Fidler
}
\keyword{internal}
