% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rudf.R
\name{.udfEnvSetUdf}
\alias{.udfEnvSetUdf}
\title{Use the udf model variable information to get the environment where
the functions exists}
\usage{
.udfEnvSetUdf(udf)
}
\arguments{
\item{udf}{modelVars$udf, integer vector with NA_integer_ for the
address of the environment where the functions exist}
}
\value{
nothing called for side effects
}
\description{
Use the udf model variable information to get the environment where
the functions exists
}
\author{
Matthew L. Fidler
}
\keyword{internal}
