% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.minfo}
\alias{.minfo}
\alias{.malert}
\alias{.mwarn}
\alias{.msuccess}
\title{Internal messaging statements}
\usage{
.minfo(text, ..., .envir = parent.frame())

.malert(text, ..., .envir = parent.frame())

.mwarn(text, ..., .envir = parent.frame())

.msuccess(text, ..., .envir = parent.frame())
}
\arguments{
\item{text}{Text}

\item{...}{Other arguments}

\item{.envir}{Environment to evaluate in}
}
\value{
Nothing, called for side effects
}
\description{
Internal messaging statements
}
\author{
Matthew L. Fidler
}
\keyword{internal}
