% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{rxGetrxode2}
\alias{rxGetrxode2}
\title{Get rxode2 model from object}
\usage{
rxGetrxode2(obj)
}
\arguments{
\item{obj}{rxode2 family of objects}
}
\value{
rxode2 model
}
\description{
Get rxode2 model from object
}
