% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tran.R
\name{rxode2parse}
\alias{rxode2parse}
\title{Internal translation to get model variables list}
\usage{
rxode2parse(
  model,
  linear = FALSE,
  linCmtSens = c("linCmtA", "linCmtB"),
  verbose = FALSE,
  code = NULL,
  envir = parent.frame()
)
}
\arguments{
\item{model}{Model (either file name or string)}

\item{linear}{boolean indicating if linear compartment model should
be generated from \code{linCmt()} (default FALSE)}

\item{linCmtSens}{Linear compartment model sensitivity type}

\item{verbose}{is a boolean indicating the type of model detected
with \code{linCmt()} parsing}

\item{code}{is a file name where the c code is written to (for
testing purposes mostly, it needs \code{rxode2} to do anything fancy)}

\item{envir}{is the environment to look for R user functions
(defaults to parent environment)}
}
\value{
A rxModelVars object that has the model variables of a
rxode2 syntax expression
}
\description{
Internal translation to get model variables list
}
\examples{
rxode2parse("a=3")
}
