% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/et.R, R/format.R, R/print.R
\name{rxEvid}
\alias{rxEvid}
\alias{as.rxEvid}
\alias{c.rxEvid}
\alias{[.rxEvid}
\alias{as.character.rxEvid}
\alias{[[.rxEvid}
\alias{units<-.rxEvid}
\alias{c.rxRateDur}
\alias{format.rxEvid}
\alias{format.rxRateDur}
\alias{print.rxEvid}
\title{EVID formatting for tibble and other places.}
\usage{
rxEvid(x)

as.rxEvid(x)

\method{c}{rxEvid}(x, ...)

\method{[}{rxEvid}(x, ...)

\method{as.character}{rxEvid}(x, ...)

\method{[[}{rxEvid}(x, ...)

\method{units}{rxEvid}(x) <- value

\method{c}{rxRateDur}(x, ...)

\method{format}{rxEvid}(x, ...)

\method{format}{rxRateDur}(x, ...)

\method{print}{rxEvid}(x, ...)
}
\arguments{
\item{x}{Item to be converted to a rxode2 EVID specification.}

\item{...}{Other parameters}

\item{value}{It will be an error to set units for evid}
}
\value{
rxEvid specification
}
\description{
This is to make an EVID more readable by non
pharmacometricians. It displays what each means and allows it to
be displayed in a tibble.
}
\examples{

rxEvid(1:7)
}
