% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxParams.R
\name{rxParams}
\alias{rxParams}
\alias{rxParams.rxode2}
\alias{rxParams.rxSolve}
\alias{rxParams.rxEt}
\alias{rxParam}
\title{Parameters specified by the model}
\usage{
rxParams(obj, ...)

\method{rxParams}{rxode2}(
  obj,
  constants = TRUE,
  ...,
  params = NULL,
  inits = NULL,
  iCov = NULL,
  keep = NULL,
  thetaMat = NULL,
  omega = NULL,
  dfSub = NULL,
  sigma = NULL,
  dfObs = NULL,
  nSub = NULL,
  nStud = NULL
)

\method{rxParams}{rxSolve}(
  obj,
  constants = TRUE,
  ...,
  params = NULL,
  inits = NULL,
  iCov = NULL,
  keep = NULL,
  thetaMat = NULL,
  omega = NULL,
  dfSub = NULL,
  sigma = NULL,
  dfObs = NULL,
  nSub = NULL,
  nStud = NULL
)

\method{rxParams}{rxEt}(
  obj,
  ...,
  params = NULL,
  inits = NULL,
  iCov = NULL,
  keep = NULL,
  thetaMat = NULL,
  omega = NULL,
  dfSub = NULL,
  sigma = NULL,
  dfObs = NULL,
  nSub = NULL,
  nStud = NULL
)

rxParam(obj, ...)
}
\arguments{
\item{obj}{rxode2 family of objects}

\item{...}{Other arguments including scaling factors for each
compartment.  This includes \code{S# =} numeric will scale a compartment
\code{#} by a dividing the compartment amount by the scale factor,
like NONMEM.}

\item{constants}{is a boolean indicting if constants should be
included in the list of parameters. Currently rxode2 parses
constants into variables in case you wish to change them
without recompiling the rxode2 model.}

\item{params}{a numeric named vector with values for every
parameter in the ODE system; the names must correspond to the
parameter identifiers used in the ODE specification;}

\item{inits}{a vector of initial values of the state variables
(e.g., amounts in each compartment), and the order in this
vector must be the same as the state variables (e.g., PK/PD
compartments);}

\item{iCov}{A data frame of individual non-time varying covariates
to combine with the \code{events} dataset.  The \code{iCov} dataset has one
covariate per ID and should match the event table}

\item{keep}{Columns to keep from either the input dataset or the
\code{iCov} dataset.  With the \code{iCov} dataset, the column
is kept once per line.  For the input dataset, if any records
are added to the data LOCF (Last Observation Carried forward)
imputation is performed.}

\item{thetaMat}{Named theta matrix.}

\item{omega}{Estimate of Covariance matrix. When omega is a list,
assume it is a block matrix and convert it to a full matrix for
simulations.  When \code{omega} is \code{NA} and you are using it with a
\code{rxode2} ui model, the between subject variability described by
the \code{omega} matrix are set to zero.}

\item{dfSub}{Degrees of freedom to sample the between subject variability matrix from the
inverse Wishart distribution (scaled) or scaled inverse chi squared distribution.}

\item{sigma}{Named sigma covariance or Cholesky decomposition of a
covariance matrix.  The names of the columns indicate
parameters that are simulated.  These are simulated for every
observation in the solved system. When \code{sigma} is \code{NA} and you are using it with a
\code{rxode2} ui model, the unexplained variability described by
the \code{sigma} matrix are set to zero.}

\item{dfObs}{Degrees of freedom to sample the unexplained variability matrix from the
inverse Wishart distribution (scaled) or scaled inverse chi squared distribution.}

\item{nSub}{Number between subject variabilities (\code{ETAs}) simulated for every
realization of the parameters.}

\item{nStud}{Number virtual studies to characterize uncertainty in estimated
parameters.}
}
\value{
When extracting the parameters from an rxode2 model, a
character vector listing the parameters in the model.
}
\description{
This returns the model's parameters that are required to solve the
ODE system, and can be used to pipe parameters into an rxode2 solve
}
\seealso{
Other Query model information:
\code{\link[=rxDfdy]{rxDfdy()}},
\code{\link[=rxInits]{rxInits()}},
\code{\link[=rxLhs]{rxLhs()}},
\code{\link[=rxModelVars]{rxModelVars()}},
\code{\link[=rxState]{rxState()}}
}
\author{
Matthew L.Fidler
}
\concept{Query model information}
