% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/collectWarnings.R
\name{.collectWarnings}
\alias{.collectWarnings}
\title{Collect warnings and just warn once.}
\usage{
.collectWarnings(expr, lst = FALSE)
}
\arguments{
\item{expr}{R expression}

\item{lst}{When \code{TRUE} return a list with
list(object,warnings) instead of issuing the warnings.
Otherwise, when \code{FALSE} issue the warnings and return the
object.}
}
\value{
The value of the expression or a list with the value of
the expression and a list of warning messages
}
\description{
Collect warnings and just warn once.
}
\author{
Matthew L. Fidler
}
