% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.rxLinCmtGen}
\alias{.rxLinCmtGen}
\title{Internal function to generate the model variables for a linCmt() model}
\usage{
.rxLinCmtGen(lenState, vars)
}
\arguments{
\item{lenState}{Length of the state}

\item{vars}{Variables in the model}
}
\value{
Model variables of expanded linCmt model
}
\description{
Internal function to generate the model variables for a linCmt() model
}
\author{
Matthew L. Fidler
}
