% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ui-getControl.R
\name{rxSetControl}
\alias{rxSetControl}
\title{rxSetControl options for UI object}
\usage{
rxSetControl(ui, control)
}
\arguments{
\item{ui}{rxode2 ui object}

\item{control}{Default value}
}
\value{
Nothing, called for side effects
}
\description{
rxSetControl options for UI object
}
\author{
Matthew L. Fidler
}
