% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/err-foceiBase.R
\name{.rxGetLambdaFromPred1AndIni}
\alias{.rxGetLambdaFromPred1AndIni}
\title{Get the lambda value based on the pred information}
\usage{
.rxGetLambdaFromPred1AndIni(env, pred1)
}
\arguments{
\item{env}{Environment that has the environment}

\item{pred1}{Single error data frame}
}
\value{
Lambda expression
}
\description{
Get the lambda value based on the pred information
}
\author{
Matthew Fidler
}
\keyword{internal}
