% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rudfui.R
\name{rxUdfUiIniDf}
\alias{rxUdfUiIniDf}
\title{Get the rxode2 iniDf of the current UI being processed (or return NULL)}
\usage{
rxUdfUiIniDf()
}
\value{
Initial \code{data.frame} being processed or \code{NULL} for nothing.
}
\description{
Get the rxode2 iniDf of the current UI being processed (or return NULL)
}
\examples{

rxUdfUiIniDf()

}
\author{
Matthew L. Fidler
}
