% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tran.R
\name{rxode2parseAssignPointerTranslation}
\alias{rxode2parseAssignPointerTranslation}
\title{This sets function gets the currently assigned function pointer assignments}
\usage{
rxode2parseAssignPointerTranslation(var)
}
\arguments{
\item{var}{List of packages where pointer assignment will be called.}
}
\value{
Nothing, called for side effects
}
\description{
This sets function gets the currently assigned function pointer assignments
}
\examples{
rxode2parseAssignPointerTranslation("rxode2")
}
\author{
Matthew L. Fidler
}
\keyword{internal}
