% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxPrune.R
\name{.rxPrune}
\alias{.rxPrune}
\title{Internal Pruning function}
\usage{
.rxPrune(x, envir = parent.frame(), strAssign = list())
}
\arguments{
\item{x}{List of quoted lines}

\item{envir}{Environment where information is stored}

\item{strAssign}{string assignment list from  model variables}
}
\value{
Pruned model code
}
\description{
Internal Pruning function
}
\author{
Matthew L. Fidler
}
\keyword{internal}
