% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rudf.R
\name{.udfEnvReset}
\alias{.udfEnvReset}
\title{Lock/Unlock environment for getting R user functions}
\usage{
.udfEnvReset(lock = TRUE)
}
\arguments{
\item{lock}{logical to see if environment to look for user defined
functions is locked.  If it is locked then environments are not
assigned.  When NULL returns lock status}
}
\value{
lock status
}
\description{
Lock/Unlock environment for getting R user functions
}
\author{
Matthew L. Fidler
}
\keyword{internal}
