% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxsolve.R
\name{rxIsStiff}
\alias{rxIsStiff}
\title{Check whether an ODE solving method is a purely stiff solver}
\usage{
rxIsStiff(method)
}
\arguments{
\item{method}{A character vector of method names or an integerish vector of
method codes (as returned by \code{\link[=odeMethodToInt]{odeMethodToInt()}}).  Vectorised.}
}
\value{
A logical vector the same length as \code{method}.  \code{TRUE} if the
corresponding method is a purely stiff solver.
}
\description{
Returns \code{TRUE} for methods that are designed exclusively for stiff systems
and will never switch to a non-stiff algorithm.  Solvers like \code{"lsoda"} and
\code{"liblsoda"} that automatically switch between stiff and non-stiff algorithms
return \code{FALSE}.
}
\details{
Stiff-only methods are:
\code{"ros4"} (13), \code{"iem"} (14), \code{"cvode"} (21), \code{"ros43"} (31), \code{"ros6"} (32),
\code{"backwardEuler"} (33), \code{"gauss6"} (34), \code{"iiic6"} (35), \code{"radauiia5"} (36),
\code{"geng5"} (37), \code{"sdirk43"} (38), \code{"bdf"} (107).
}
\examples{
rxIsStiff("bdf")                           # TRUE
rxIsStiff("lsoda")                         # FALSE (switches)
rxIsStiff("liblsoda")                      # FALSE (switches)
rxIsStiff(c("cvode", "dop853", "ros43"))   # TRUE FALSE TRUE

}
\seealso{
\code{\link[=rxIsNonStiff]{rxIsNonStiff()}}, \code{\link[=rxIsImplicit]{rxIsImplicit()}}, \code{\link[=odeMethodToInt]{odeMethodToInt()}}
}
