% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{rxLhs}
\alias{rxLhs}
\title{Left handed Variables}
\usage{
rxLhs(obj)
}
\arguments{
\item{obj}{rxode2 family of objects}
}
\value{
a character vector listing the calculated parameters
}
\description{
This returns the model calculated variables
}
\seealso{
\code{\link{rxode2}}

Other Query model information:
\code{\link[=rxDfdy]{rxDfdy()}},
\code{\link[=rxInits]{rxInits()}},
\code{\link[=rxModelVars]{rxModelVars()}},
\code{\link[=rxParams]{rxParams()}},
\code{\link[=rxState]{rxState()}}
}
\author{
Matthew L.Fidler
}
\concept{Query model information}
