% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assert.R
\name{assertCompartmentExists}
\alias{assertCompartmentExists}
\alias{testCompartmentExists}
\title{Verify that the compartment exists in a model}
\usage{
assertCompartmentExists(ui, x)

testCompartmentExists(ui, x)
}
\arguments{
\item{ui}{is the model to test}

\item{x}{The value to test (can be a vector of strings)}
}
\value{
the value of the compartment that exists; if it is a vector
returns the first item that matches
}
\description{
Verify that the compartment exists in a model
}
\section{Functions}{
\itemize{
\item \code{testCompartmentExists()}: Test if compartment exists

}}
\seealso{
Other Assertions:
\code{\link[=assertCompartmentName]{assertCompartmentName()}},
\code{\link[=assertCompartmentNew]{assertCompartmentNew()}},
\code{\link[=assertRxUi]{assertRxUi()}},
\code{\link[=assertVariableExists]{assertVariableExists()}},
\code{\link[=assertVariableNew]{assertVariableNew()}},
\code{\link[=testIniDf]{testIniDf()}},
\code{\link[=testRxUnbounded]{testRxUnbounded()}}
}
\author{
Matthew Fidler & Bill Denney
}
\concept{Assertions}
