% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rudfui.R
\name{rxUdfUiFlag}
\alias{rxUdfUiFlag}
\title{Is the expression actually a flag that can be used in the rxUdfUi
functions?}
\usage{
rxUdfUiFlag(expr, arg = "arg", funName = "fun", env = baseenv())
}
\arguments{
\item{expr}{expression to evaluate}

\item{arg}{argument name for error messages for the argument name
in the \code{rxUdfUi} extension when the expression is not logical}

\item{funName}{function name for the error message for the argument
name in the \code{rxUdfUi} extension when the expression is not
logical.}

\item{env}{the environment in which to evaluate the expression (in case it is numeric)}
}
\value{
logical value of the expression if it can be evaluated to a
scalar \code{TRUE}/\code{FALSE} value, otherwise an error is thrown.
}
\description{
This is useful when writing replacement
UI functions
}
\author{
Matthew L. Fidler
}
