% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/piping.R
\name{.quoteCallInfoLines}
\alias{.quoteCallInfoLines}
\title{Returns quoted call information}
\usage{
.quoteCallInfoLines(callInfo, envir = parent.frame(), iniDf = NULL)
}
\arguments{
\item{callInfo}{Call information}

\item{envir}{Environment for evaluation (if needed)}

\item{iniDf}{The parent model \code{iniDf} when piping in a \code{ini} block
(\code{NULL} otherwise)}
}
\value{
Quote call information.  for \code{name=expression}, change to
\code{name<-expression} in quoted call list. For expressions that are
within brackets ie \code{{}}, unlist the brackets as if they were
called in one single sequence.
}
\description{
Returns quoted call information
}
\author{
Matthew L. Fidler
}
