% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/err-foceiBase.R
\name{.rxGetVarianceForErrorType}
\alias{.rxGetVarianceForErrorType}
\title{Get Variance for error type}
\usage{
.rxGetVarianceForErrorType(env, pred1)
}
\arguments{
\item{env}{Environment}

\item{pred1}{Pred for one end-point}
}
\value{
Variance error type
}
\description{
Get Variance for error type
}
\author{
Matthew L. Fidler
}
\keyword{internal}
