% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxrandom.R
\name{.rxGetSeed}
\alias{.rxGetSeed}
\title{Get the random seed for the session}
\usage{
.rxGetSeed()
}
\value{
A list with the following components:
\itemize{
\item \code{seed} The random seed for the session.  This is the value of
\code{.Random.seed} in the global environment.
\item \code{kind} The random number generator kind for the session.  This is
the value of \code{RNGkind()} for the session.
\item \code{rxseed} The rxode2 random seed for the session.  This
is the value of \code{rxGetSeed()} for the session.
}
}
\description{
Get the random seed for the session
}
\author{
Matthew L. Fidler
}
\keyword{internal}
