% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{rxSolveSetup}
\alias{rxSolveSetup}
\title{See if the memory is installed for a solve}
\usage{
rxSolveSetup()
}
\value{
boolean saying if the memnory is currently free for rxode2
}
\description{
See if the memory is installed for a solve
}
\author{
Matthew L. Fidler
}
\keyword{internal}
