% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxUiGet.R, R/rxsolve.R
\name{rxUiDeparse}
\alias{rxUiDeparse}
\alias{rxUiDeparse.lotriFix}
\alias{rxUiDeparse.default}
\alias{rxUiDeparse.rxControl}
\title{This is a generic function for deparsing certain objects when
printing out a rxode2 object.  Currently it is used for any meta-information}
\usage{
rxUiDeparse(object, var)

\method{rxUiDeparse}{lotriFix}(object, var)

\method{rxUiDeparse}{default}(object, var)

\method{rxUiDeparse}{rxControl}(object, var)
}
\arguments{
\item{object}{object to be deparsed}

\item{var}{variable name to be assigned}
}
\value{
parsed R expression that can be used for printing and
\code{as.function()} calls.
}
\description{
This is a generic function for deparsing certain objects when
printing out a rxode2 object.  Currently it is used for any meta-information

rxUiDeparse.rxControl(rxControl(covsInterpolation="linear", method="dop853",
naInterpolation="nocb", keepInterpolation="nocb", sigmaXform="variance",
omegaXform="variance", returnType="data.frame", sumType="fsum", prodType="logify"),
"ctl")
}
\examples{

mat <- matrix(c(1, 0.1, 0.1, 1), 2, 2, dimnames=list(c("a", "b"), c("a", "b")))

rxUiDeparse(matrix(c(1, 0.1, 0.1, 1), 2, 2, dimnames=list(c("a", "b"), c("a", "b"))), "x")
}
\author{
Matthew L. Fidler
}
