% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/piping-ini.R
\name{.iniHandleAppend}
\alias{.iniHandleAppend}
\title{Reorder rows in iniDf}
\usage{
.iniHandleAppend(expr, rxui, envir, append)
}
\arguments{
\item{expr}{Expression for parsing}

\item{rxui}{User interface function}

\item{envir}{Environment for parsing}

\item{append}{Reorder theta parameters.  \code{NULL} means no change to
parameter order.  A parameter name (as a character string) means to put the
new parameter after the named parameter.  A number less than or equal to
zero means to put the parameter at the beginning of the list.  A number
greater than the last parameter number means to put the parameter at the
end of the list.}
}
\value{
Nothing, called for side effects
}
\description{
Reorder rows in iniDf
}
\keyword{internal}
