% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rudf.R
\name{.udfEnvSet}
\alias{.udfEnvSet}
\title{Setup the UDF environment (for querying user defined funtions)}
\usage{
.udfEnvSet(env)
}
\arguments{
\item{env}{environment where user defined functions are queried. If NULL return current environment}
}
\value{
environment
}
\description{
Setup the UDF environment (for querying user defined funtions)
}
\author{
Matthew L. Fidler
}
\keyword{internal}
