% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rudf.R
\name{.rxC}
\alias{.rxC}
\title{Return the C code of an internal function}
\usage{
.rxC(fun)
}
\arguments{
\item{fun}{is the string of a function that you wish to get the C
code for}
}
\value{
C code if found (as a string) or NULL if not found
}
\description{
Return the C code of an internal function
}
\author{
Matthew Fider
}
\keyword{internal}
