% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/piping-model.R
\name{.modelHandleModelLines}
\alias{.modelHandleModelLines}
\title{Handle model lines}
\usage{
.modelHandleModelLines(
  modelLines,
  rxui,
  modifyIni = FALSE,
  append = NULL,
  auto = getOption("rxode2.autoVarPiping", TRUE),
  cov = NULL,
  envir
)
}
\arguments{
\item{modelLines}{The model lines that are being considered}

\item{rxui}{The rxode2 UI object}

\item{modifyIni}{Should the ini({}) be considered}

\item{append}{This is a boolean to determine if the lines are
appended in piping.  The possible values for this is:
\itemize{
\item \code{TRUE} which is when the lines are appended to the model instead of replaced
\item \code{FALSE} when the lines are replaced in the model (default)
\item \code{NA} is when the lines are pre-pended to the model instead of replaced
\item \verb{lhs expression}, which will append the lines after the last observed line of the expression \code{lhs}
}}

\item{auto}{This boolean tells if piping automatically selects the
parameters should be characterized as a population parameter,
between subject variability, or a covariate.  When \code{TRUE} this
automatic selection occurs.  When \code{FALSE} this automatic
selection is turned off and everything is added as a covariate
(which can be promoted to a parameter with the \code{ini} statement).
By default this is \code{TRUE}, but it can be changed by
\code{options(rxode2.autoVarPiping=FALSE)}.}

\item{cov}{is a character vector of variables that should be
assumed to be covariates.  This will override automatic promotion
to a population parameter estimate (or an eta)}

\item{envir}{Environment for evaluation}
}
\value{
New UI
}
\description{
Handle model lines
}
\author{
Matthew L. Fidler
}
