% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{boxCox}
\alias{boxCox}
\alias{boxCoxInv}
\alias{yeoJohnson}
\alias{yeoJohnsonInv}
\title{boxCox/yeoJohnson and inverse boxCox/yeoJohnson functions}
\usage{
boxCox(x, lambda = 1)

boxCoxInv(x, lambda = 1)

yeoJohnson(x, lambda = 1)

yeoJohnsonInv(x, lambda = 1)
}
\arguments{
\item{x}{input value(s) to transform}

\item{lambda}{lambda value for the transformation}
}
\value{
values from boxCox and boxCoxInv
}
\description{
boxCox/yeoJohnson and inverse boxCox/yeoJohnson functions
}
\examples{

boxCox(10, 0.5)

boxCoxInv(4.32, 0.5)

yeoJohnson(10, 0.5)

yeoJohnsonInv(4.32, 0.5)

}
