% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{rxExpandSens3_}
\alias{rxExpandSens3_}
\title{Expand third order sensitivity}
\usage{
rxExpandSens3_(state, s1, s2, s3)
}
\arguments{
\item{state}{is the state to expand}

\item{s1}{is the newly added (outer) sensitivity variable}

\item{s2}{is the second sensitivity variable}

\item{s3}{is the third sensitivity variable}
}
\value{
data frame of symengine third order sensitivity code
}
\description{
Builds the symengine code for the third order forward sensitivity
d/dt(d^3 state / d s1 d s2 d s3) by total-differentiating the stored
second order sensitivity RHS (for the pair s2,s3) with respect to the
newly added variable s1.  Mirrors \code{rxExpandSens2_}; assumes the
first and second order sensitivities (and the Jacobian) are already
loaded in the symengine \code{model} environment.
}
\keyword{internal}
