% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{rxUnloadAll}
\alias{rxUnloadAll}
\title{Unloads all rxode2 compiled DLLs}
\usage{
rxUnloadAll(set = TRUE)
}
\arguments{
\item{set}{If specified and \code{TRUE}, unloads all rxode2 dlls.  If
specified and \code{FALSE}, block a simple \code{rxUnloadAll()} will not
actually unload all dlls (helps with CRAN ASAN check)}
}
\value{
boolean telling if \code{rxUnloadAll()} completed the unloading procedure
}
\description{
Unloads all rxode2 compiled DLLs
}
\examples{

# print(rxUnloadAll())
}
