% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rudf.R
\name{.udfExists}
\alias{.udfExists}
\title{See if the UI function exists in given environment.}
\usage{
.udfExists(fun, nargs, envir, doList = TRUE)
}
\arguments{
\item{fun}{Function to check}

\item{nargs}{Number of args to check}

\item{envir}{Environment to check}

\item{doList}{A boolean to see if the functions in .udfEnv$fun
should be checked too.  By default TRUE, but this is called
recursively for each function (and set to FALSE)}
}
\value{
logical declaring if the udf function exists in this environment
}
\description{
If other functions have been declared, make sure they exist too.
}
\author{
Matthew L. Fidler
}
\keyword{internal}
