% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/err-foceiBase.R
\name{.rxGetPredictionFTransform}
\alias{.rxGetPredictionFTransform}
\title{Get the prediction transformation}
\usage{
.rxGetPredictionFTransform(env, pred1, yj)
}
\arguments{
\item{env}{Environment for the parsed model}

\item{pred1}{The \code{data.frame} of the current error}

\item{yj}{The transformation number for the current error}
}
\value{
The transformation expression
}
\description{
Get the prediction transformation
}
\author{
Matthew Fidler
}
\keyword{internal}
