% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxode-options.R
\name{rxSyncOptions}
\alias{rxSyncOptions}
\title{Sync options with rxode2 variables}
\usage{
rxSyncOptions(setDefaults = c("none", "permissive", "strict"))
}
\arguments{
\item{setDefaults}{This will setup rxode2's default solving options with the following options:
\itemize{
\item \code{"none"} leave the options alone
\item \code{"permissive"} This is a permissive option set similar to R language specifications.
\item \code{"strict"} This is a strict option set similar to the original
rxode2(). It requires semicolons at the end of lines and equals for
assignment
}}
}
\value{
nothing; called for side effects
}
\description{
Sync options with rxode2 variables
}
\author{
Matthew L. Fidler
}
