% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxode2.R
\name{.useUtf}
\alias{.useUtf}
\title{Internal function to figure out if this session supports Unicode}
\usage{
.useUtf()
}
\value{
boolean indicating if this session supports Unicode
}
\description{
Internal function to figure out if this session supports Unicode
}
\keyword{internal}
