% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fixef.R
\name{fixef.rxUi}
\alias{fixef.rxUi}
\alias{fixef.function}
\title{Extract fixed effects from an rxode2 model (rxUi)}
\usage{
fixef.rxUi(object, ...)

fixef.function(object, ...)
}
\arguments{
\item{object}{rxode2 ui model or a function that evaluates to one}

\item{...}{other parameters (currently ignored)}
}
\value{
named numeric vector of the fixed-effect (theta) estimates
}
\description{
Extract fixed effects from an rxode2 model (rxUi)
}
\examples{
one.compartment <- function() {
  ini({
    tka <- 0.45
    tcl <- 1
    tv <- 3.45
    eta.ka ~ 0.6
  })
  model({
    ka <- exp(tka + eta.ka)
    cl <- exp(tcl)
    v <- exp(tv)
    d/dt(depot) <- -ka * depot
    d/dt(center) <- ka * depot - cl / v * center
    cp <- center / v
  })
}

ui <- one.compartment()
nlme::fixef(ui)
}
\author{
Matthew L. Fidler
}
\keyword{internal}
