% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxrandom.R
\name{.expandPars}
\alias{.expandPars}
\title{Expand parameters}
\usage{
.expandPars(object, params, events, control)
}
\arguments{
\item{object}{rxode2 model variables object}

\item{params}{parameters to expand}

\item{events}{event table to help with the expansion}

\item{control}{control structure to help with the parameter generation}
}
\value{
Expanded parameters for simulation
}
\description{
Expand parameters
}
\author{
Matthew L. Fidler
}
\keyword{internal}
