% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/err.R
\name{.getErrDist}
\alias{.getErrDist}
\alias{.getNamedArgumentsToPredDf}
\title{Get defined residual information for parsing elsewhere}
\usage{
.getErrDist()

.getNamedArgumentsToPredDf()
}
\value{
A list of error distribution information
}
\description{
Get defined residual information for parsing elsewhere
}
\author{
Matthew L. Fidler
}
\keyword{internal}
