% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.rxLinNcmt}
\alias{.rxLinNcmt}
\title{Get the number of linear compartments}
\usage{
.rxLinNcmt(obj)
}
\arguments{
\item{obj}{rxode2 object}
}
\value{
A named integer vector with the number of linear compartments (numLin, numLinSens and depotLin)
}
\description{
Get the number of linear compartments
}
\author{
Matthew L. Fidler
}
\keyword{internal}
