% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/err-sim.R, R/rxUiGet.R
\name{rxUiGet.cmtLines}
\alias{rxUiGet.cmtLines}
\alias{rxUiGet.dvidLine}
\alias{rxUiGet.paramsLine}
\alias{rxUiGet.interpLines}
\alias{rxUiGet.splitDose}
\alias{rxUiGet.splitDoseLines}
\alias{rxUiGet.simulationSigma}
\alias{rxUiGet.simulationModel}
\alias{rxUiGet.symengineModelNoPrune}
\alias{rxUiGet.symengineModelPrune}
\alias{rxUiGet.simulationIniModel}
\alias{rxUiGet}
\alias{rxUiGet.levels}
\alias{rxUiGet.state}
\alias{rxUiGet.stateDf}
\alias{rxUiGet.statePropDf}
\alias{rxUiGet.props}
\alias{rxUiGet.theta}
\alias{rxUiGet.lstChr}
\alias{rxUiGet.omega}
\alias{rxUiGet.funTxt}
\alias{rxUiGet.allCovs}
\alias{rxUiGet.muRefTable}
\alias{rxUiGet.multipleEndpoint}
\alias{rxUiGet.funPrint}
\alias{rxUiGet.fun}
\alias{rxUiGet.funPartsDigest}
\alias{rxUiGet.md5}
\alias{rxUiGet.sha1}
\alias{rxUiGet.ini}
\alias{rxUiGet.iniFun}
\alias{rxUiGet.modelFun}
\alias{rxUiGet.model}
\alias{rxUiGet.modelDesc}
\alias{rxUiGet.thetaLower}
\alias{rxUiGet.thetaUpper}
\alias{rxUiGet.lhsVar}
\alias{rxUiGet.varLhs}
\alias{rxUiGet.lhsEta}
\alias{rxUiGet.lhsTheta}
\alias{rxUiGet.lhsCov}
\alias{rxUiGet.etaLhs}
\alias{rxUiGet.thetaLhs}
\alias{rxUiGet.covLhs}
\alias{rxUiGet.levelLhs}
\alias{rxUiGet.default}
\title{S3 for getting information from UI model}
\usage{
\method{rxUiGet}{cmtLines}(x, ...)

\method{rxUiGet}{dvidLine}(x, ...)

\method{rxUiGet}{paramsLine}(x, ...)

\method{rxUiGet}{interpLines}(x, ...)

\method{rxUiGet}{splitDose}(x, ...)

\method{rxUiGet}{splitDoseLines}(x, ...)

\method{rxUiGet}{simulationSigma}(x, ...)

\method{rxUiGet}{simulationModel}(x, ...)

\method{rxUiGet}{symengineModelNoPrune}(x, ...)

\method{rxUiGet}{symengineModelPrune}(x, ...)

\method{rxUiGet}{simulationIniModel}(x, ...)

rxUiGet(x, ...)

\method{rxUiGet}{levels}(x, ...)

\method{rxUiGet}{state}(x, ...)

\method{rxUiGet}{stateDf}(x, ...)

\method{rxUiGet}{statePropDf}(x, ...)

\method{rxUiGet}{props}(x, ...)

\method{rxUiGet}{theta}(x, ...)

\method{rxUiGet}{lstChr}(x, ...)

\method{rxUiGet}{omega}(x, ...)

\method{rxUiGet}{funTxt}(x, ...)

\method{rxUiGet}{allCovs}(x, ...)

\method{rxUiGet}{muRefTable}(x, ...)

\method{rxUiGet}{multipleEndpoint}(x, ...)

\method{rxUiGet}{funPrint}(x, ...)

\method{rxUiGet}{fun}(x, ...)

\method{rxUiGet}{funPartsDigest}(x, ...)

\method{rxUiGet}{md5}(x, ...)

\method{rxUiGet}{sha1}(x, ...)

\method{rxUiGet}{ini}(x, ...)

\method{rxUiGet}{iniFun}(x, ...)

\method{rxUiGet}{modelFun}(x, ...)

\method{rxUiGet}{model}(x, ...)

\method{rxUiGet}{modelDesc}(x, ...)

\method{rxUiGet}{thetaLower}(x, ...)

\method{rxUiGet}{thetaUpper}(x, ...)

\method{rxUiGet}{lhsVar}(x, ...)

\method{rxUiGet}{varLhs}(x, ...)

\method{rxUiGet}{lhsEta}(x, ...)

\method{rxUiGet}{lhsTheta}(x, ...)

\method{rxUiGet}{lhsCov}(x, ...)

\method{rxUiGet}{etaLhs}(x, ...)

\method{rxUiGet}{thetaLhs}(x, ...)

\method{rxUiGet}{covLhs}(x, ...)

\method{rxUiGet}{levelLhs}(x, ...)

\method{rxUiGet}{default}(x, ...)
}
\arguments{
\item{x}{list of (UIenvironment, exact).  UI environment is the
parsed function for rxode2.  \code{exact} is a boolean that says if an
exact match is required.}

\item{...}{Other arguments}
}
\value{
value that was requested from the UI object
}
\description{
S3 for getting information from UI model
}
\author{
Matthew Fidler
}
