% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rudfui.R
\name{rxUdfUiIsValue}
\alias{rxUdfUiIsValue}
\title{Is the expression actually equal to a value?}
\usage{
rxUdfUiIsValue(expr, value, env = baseenv())
}
\arguments{
\item{expr}{the R language expression to evaluate}

\item{value}{the value to compare against}

\item{env}{the environment in which to evaluate the expression}
}
\value{
TRUE if the expression evaluates to a scalar value equal to
the specified value, FALSE otherwise
}
\description{
This is used to determine if the location and scale parameters are
actually equal to their default values, which allows for more
efficient translation to expit expressions when possible.
}
\author{
Matthew L. Fidler
}
