% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/softplus.R
\name{softplus}
\alias{softplus}
\title{Softplus Activation Function}
\usage{
softplus(x)
}
\arguments{
\item{x}{numeric vector}
}
\value{
numeric vector
}
\description{
Softplus Activation Function
}
\examples{

softplus(c(-1, 0, 1, 2))

\donttest{
# You can use rxode2 too:

r <- rxode2({
 s <- softplus(time)
})

e <- et(c(-1, 0, 1, 2))

rxSolve(r, e)
}

}
\seealso{
Other Activation Functions:
\code{\link[=ELU]{ELU()}},
\code{\link[=GELU]{GELU()}},
\code{\link[=PReLU]{PReLU()}},
\code{\link[=ReLU]{ReLU()}},
\code{\link[=SELU]{SELU()}},
\code{\link[=Swish]{Swish()}},
\code{\link[=dELU]{dELU()}},
\code{\link[=dGELU]{dGELU()}},
\code{\link[=dPReLU]{dPReLU()}},
\code{\link[=dReLU]{dReLU()}},
\code{\link[=dSELU]{dSELU()}},
\code{\link[=dSwish]{dSwish()}},
\code{\link[=dlReLU]{dlReLU()}},
\code{\link[=dsoftplus]{dsoftplus()}},
\code{\link[=lReLU]{lReLU()}}
}
\author{
Matthew L. Fidler
}
\concept{Activation Functions}
