% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/piping-update.R
\name{update.rxUi}
\alias{update.rxUi}
\title{Update for rxUi}
\usage{
\method{update}{rxUi}(object, ..., envir = parent.frame())
}
\arguments{
\item{object}{rxode2 UI object}

\item{...}{Lines to update}

\item{envir}{Environment for evaluating ini({}) style calls}
}
\value{
a new rxode2 updated UI object
}
\description{
Update for rxUi
}
