% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/err-foceiBase.R
\name{.rxGetLowBoundaryPred1AndIni}
\alias{.rxGetLowBoundaryPred1AndIni}
\title{Get the lower boundary condition when the transformation requires it}
\usage{
.rxGetLowBoundaryPred1AndIni(env, pred1)
}
\arguments{
\item{env}{Environment for the parsed model}

\item{pred1}{The \code{data.frame} of the current error}
}
\value{
Lower Boundary
}
\description{
Get the lower boundary condition when the transformation requires it
}
\author{
Matthew Fidler
}
\keyword{internal}
