% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rudf.R
\name{.extraCnow}
\alias{.extraCnow}
\title{Get the extraCnow for compiling}
\usage{
.extraCnow()
}
\value{
string of extraC information
}
\description{
Get the extraCnow for compiling
}
\author{
Matthew L. Fidler
}
\keyword{internal}
