% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assert.R
\name{assertVariableExists}
\alias{assertVariableExists}
\alias{testVariableExists}
\title{Assert a variable exists in the model}
\usage{
assertVariableExists(ui, x)

testVariableExists(ui, x)
}
\arguments{
\item{ui}{rxode2 ui model}

\item{x}{does the \code{x} variable exist in the model.  If it is a
vector of variable check to see if any exists, but all must be
valid nlmixr2 variable names}
}
\value{
variable that matches, in the case of multiple variables,
the first that matches.  If nothing matches return error
}
\description{
Assert a variable exists in the model
}
\section{Functions}{
\itemize{
\item \code{testVariableExists()}: Test if variable exists

}}
\seealso{
Other Assertions:
\code{\link[=assertCompartmentExists]{assertCompartmentExists()}},
\code{\link[=assertCompartmentName]{assertCompartmentName()}},
\code{\link[=assertCompartmentNew]{assertCompartmentNew()}},
\code{\link[=assertRxUi]{assertRxUi()}},
\code{\link[=assertVariableNew]{assertVariableNew()}},
\code{\link[=testIniDf]{testIniDf()}},
\code{\link[=testRxUnbounded]{testRxUnbounded()}}
}
\author{
Matthew L. Fidler
}
\concept{Assertions}
