% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rudfui.R
\name{rxUdfUiMv}
\alias{rxUdfUiMv}
\title{Return the model variables that is being processed or setup model
variables for processing}
\usage{
rxUdfUiMv(value)
}
\arguments{
\item{value}{when specified, this assigns the model variables to be
processed, or resets it by assigning it to be \code{NULL}.}
}
\value{
value of the \code{modelVariables} being processed or \code{NULL}.
}
\description{
Return the model variables that is being processed or setup model
variables for processing
}
\examples{

rxUdfUiMv()

}
\seealso{
Other User functions:
\code{\link[=linMod]{linMod()}},
\code{\link[=rxUdfUiControl]{rxUdfUiControl()}},
\code{\link[=rxUdfUiData]{rxUdfUiData()}},
\code{\link[=rxUdfUiEst]{rxUdfUiEst()}},
\code{\link[=rxUdfUiIniLhs]{rxUdfUiIniLhs()}},
\code{\link[=rxUdfUiNum]{rxUdfUiNum()}},
\code{\link[=rxUdfUiParsing]{rxUdfUiParsing()}}
}
\author{
Matthew L. Fidler
}
\concept{User functions}
