% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ui-getControl.R
\name{rxAssignControlValue}
\alias{rxAssignControlValue}
\title{Assign Control Variable}
\usage{
rxAssignControlValue(ui, option, value)
}
\arguments{
\item{ui}{rxode2 ui function}

\item{option}{Option name in the control to modify}

\item{value}{Value of control to modify}
}
\value{
Nothing; called for the side effects
}
\description{
Assign Control Variable
}
\author{
Matthew L. Fidler
}
