% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rstudio.R
\name{.rstudioComplete}
\alias{.rstudioComplete}
\title{Determine if Rstudio completion 2025+ is running}
\usage{
.rstudioComplete()
}
\value{
boolean, is Rstudio completion 2025+ running
}
\description{
Determine if Rstudio completion 2025+ is running
}
\examples{
# Determines if rstudio is running completion
.rstudioComplete()
}
\author{
Matthew L. Fidler
}
\keyword{internal}
