% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxUiGet.R
\name{rxUiDevelop}
\alias{rxUiDevelop}
\title{rxUiDevelop - Enable/Disable rxUi development.  Here all $ completions are given}
\usage{
rxUiDevelop(enable = TRUE)
}
\arguments{
\item{enable}{boolean to enable/disable rxUi development mode.}
}
\value{
nothing, called for side effects
}
\description{
rxUiDevelop - Enable/Disable rxUi development.  Here all $ completions are given
}
\examples{

rxUiDevelop(TRUE)
rxUiDevelop(FALSE)
}
\author{
Matthew L. Fidler
}
