% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rudf.R
\name{.udfMd5Info}
\alias{.udfMd5Info}
\title{Get the udf strings for creating model md5}
\usage{
.udfMd5Info()
}
\value{
string vector
}
\description{
Get the udf strings for creating model md5
}
\author{
Matthew L. Fidler
}
\keyword{internal}
