% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/piping-ini.R
\name{.iniGetAppendArg}
\alias{.iniGetAppendArg}
\title{This gets the append arg for the ini({}) piping}
\usage{
.iniGetAppendArg(f, s)
}
\arguments{
\item{f}{this is the \code{try(force(append))} argument,}

\item{s}{this is the \code{as.character(substitute(append))} argument}
}
\value{
corrected ini piping argument

This is exported for creating new ini methods that have the same
requirements for piping
}
\description{
This gets the append arg for the ini({}) piping
}
\author{
Matthew L. Fidler
}
\keyword{internal}
