% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{rxExpandFEta_}
\alias{rxExpandFEta_}
\title{Expand d(f)/d(eta)}
\usage{
rxExpandFEta_(state, neta, pred, isTheta = FALSE)
}
\arguments{
\item{state}{is the state to expand}

\item{neta}{is the number of etas}

\item{pred}{type of prediction}

\item{isTheta}{logical, is the expansion actually for thetas instead of etas}
}
\value{
String of symengine expressions to evaluate to calculate df/deta
}
\description{
Expand d(f)/d(eta)
}
\keyword{internal}
