% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/linCmt.R
\name{.solComp2}
\alias{.solComp2}
\title{Calculate the lambdas and coefficients of the two compartment model}
\usage{
.solComp2(k10, k12, k21)
}
\arguments{
\item{k10}{elimination rate}

\item{k12}{rate from central to peripheral compartment}

\item{k21}{rate from peripheral to central compartment}
}
\value{
List with \code{L} vector and matrices \code{C1} and \code{C2}
}
\description{
Calculate the lambdas and coefficients of the two compartment model
}
\examples{
.solComp2(k10=0.1, k12=3, k21=1)
}
\author{
Matthew L. Fidler based on \code{wnl} package/paper, implemented
in C/C++
}
\keyword{internal}
