% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ui-rename.R
\name{.assertRenameErrorModelLine}
\alias{.assertRenameErrorModelLine}
\title{This function asserts the requested rename makes sense}
\usage{
.assertRenameErrorModelLine(line, vars)
}
\arguments{
\item{line}{quoted call information line}

\item{vars}{Variables contained within the mdel}
}
\value{
list(new, old, newChar, oldChar)
}
\description{
It returns the new expression, old expression, new variable and old
variable
}
\author{
Matthew L. Fidler
}
\keyword{internal}
