% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rudfui.R
\name{rxUdfUiEst}
\alias{rxUdfUiEst}
\title{Return the current estimation method for the UI processing}
\usage{
rxUdfUiEst(value)
}
\arguments{
\item{value}{when specified, this assigns the character value of
the estimation method or NULL if there is nothing being estimated}
}
\value{
value of the estimation method being processed or NULL
}
\description{
Return the current estimation method for the UI processing
}
\examples{

rxUdfUiEst()

}
\seealso{
Other User functions:
\code{\link[=linMod]{linMod()}},
\code{\link[=rxUdfUiControl]{rxUdfUiControl()}},
\code{\link[=rxUdfUiData]{rxUdfUiData()}},
\code{\link[=rxUdfUiIniLhs]{rxUdfUiIniLhs()}},
\code{\link[=rxUdfUiMv]{rxUdfUiMv()}},
\code{\link[=rxUdfUiNum]{rxUdfUiNum()}},
\code{\link[=rxUdfUiParsing]{rxUdfUiParsing()}}
}
\author{
Matthew L. Fidler
}
\concept{User functions}
