% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getBaseSim.R
\name{getBaseSymengineModel}
\alias{getBaseSymengineModel}
\title{Get the symengine for loading into symengine with \code{rxS()}}
\usage{
getBaseSymengineModel(obj)
}
\arguments{
\item{obj}{Object}
}
\value{
Simulation model ready to load into symeng
}
\description{
Get the symengine for loading into symengine with \code{rxS()}
}
\author{
Matthew L. Fidler
}
\keyword{internal}
