% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assert.R
\name{testIniDf}
\alias{testIniDf}
\alias{assertIniDf}
\title{This function tests if this object is a iniDf as needed by the UI}
\usage{
testIniDf(iniDf)

assertIniDf(iniDf, extra = "", .var.name = .vname(iniDf), null.ok = FALSE)
}
\arguments{
\item{iniDf}{the object to test if it is a rxode2 ui \code{iniDf} data.frame}

\item{extra}{information to append to the error message}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link[checkmate]{vname}}.}

\item{null.ok}{[\code{logical(1)}]\cr
If set to \code{TRUE}, \code{x} may also be \code{NULL}.
In this case only a type check of \code{x} is performed, all additional checks are disabled.}
}
\value{
boolean, indicating if the object is a valid initialization data frame
}
\description{
This function tests if this object is a iniDf as needed by the UI
}
\section{Functions}{
\itemize{
\item \code{assertIniDf()}: Assert that the object is a valid rxode2 ui initialization data frame

}}
\examples{
testIniDf(TRUE)
}
\seealso{
Other Assertions:
\code{\link[=assertCompartmentExists]{assertCompartmentExists()}},
\code{\link[=assertCompartmentName]{assertCompartmentName()}},
\code{\link[=assertCompartmentNew]{assertCompartmentNew()}},
\code{\link[=assertRxUi]{assertRxUi()}},
\code{\link[=assertVariableExists]{assertVariableExists()}},
\code{\link[=assertVariableNew]{assertVariableNew()}},
\code{\link[=testRxUnbounded]{testRxUnbounded()}}
}
\author{
Matthew L. Fidler
}
\concept{Assertions}
