% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reexport.R
\docType{import}
\name{reexports}
\alias{reexports}
\alias{scale_type}
\alias{ggplot}
\alias{aes}
\alias{geom_line}
\alias{facet_wrap}
\alias{ylab}
\alias{xlab}
\alias{waiver}
\alias{lotri}
\alias{label_value}
\alias{label_both}
\alias{label_context}
\alias{label_wrap_gen}
\alias{scale_x_discrete}
\alias{scale_y_discrete}
\alias{scale_x_continuous}
\alias{scale_y_continuous}
\alias{scale_x_date}
\alias{scale_y_date}
\alias{expand_limits}
\title{Objects exported from other packages}
\value{
Inherited from parent routine
}
\keyword{internal}
\description{
These objects are imported from other packages. Follow the links
below to see their documentation.

\describe{
  \item{ggplot2}{\code{\link[ggplot2:aes]{aes()}}, \code{\link[ggplot2:expand_limits]{expand_limits()}}, \code{\link[ggplot2:facet_wrap]{facet_wrap()}}, \code{\link[ggplot2:geom_line]{geom_line()}}, \code{\link[ggplot2:ggplot]{ggplot()}}, \code{\link[ggplot2:label_both]{label_both()}}, \code{\link[ggplot2:label_context]{label_context()}}, \code{\link[ggplot2:label_context]{label_context()}}, \code{\link[ggplot2:label_value]{label_value()}}, \code{\link[ggplot2:label_wrap_gen]{label_wrap_gen()}}, \code{\link[ggplot2:scale_type]{scale_type()}}, \code{\link[ggplot2:scale_x_continuous]{scale_x_continuous()}}, \code{\link[ggplot2:scale_x_date]{scale_x_date()}}, \code{\link[ggplot2:scale_x_discrete]{scale_x_discrete()}}, \code{\link[ggplot2:scale_y_continuous]{scale_y_continuous()}}, \code{\link[ggplot2:scale_y_date]{scale_y_date()}}, \code{\link[ggplot2:scale_y_discrete]{scale_y_discrete()}}, \code{\link[ggplot2:waiver]{waiver()}}, \code{\link[ggplot2:xlab]{xlab()}}, \code{\link[ggplot2:ylab]{ylab()}}}

  \item{lotri}{\code{\link[lotri:lotri]{lotri()}}}
}}

