% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{rxDll}
\alias{rxDll}
\title{Return the DLL associated with the rxode2 object}
\usage{
rxDll(obj)
}
\arguments{
\item{obj}{A rxode2 family of objects or a character string of the
model specification or location of a file with a model
specification.}
}
\value{
a path of the library
}
\description{
This will return the dynamic load library or shared object used to
run the C code for rxode2.
}
\author{
Matthew L.Fidler
}
\keyword{internal}
