% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.convertId}
\alias{.convertId}
\title{Convert a factor/char to an id}
\usage{
.convertId(a)
}
\arguments{
\item{a}{value to convert to an id}
}
\value{
id factor
}
\description{
Convert a factor/char to an id
}
\examples{
.convertId("a")
}
\author{
Matthew L. Fidler
}
\keyword{internal}
