% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rudf.R
\name{.extraC}
\alias{.extraC}
\title{Generate extraC information for rxode2 models}
\usage{
.extraC(extraC = NULL)
}
\arguments{
\item{extraC}{Additional extraC from rxode2 compile optioioins}
}
\value{
Nothing, called for side effects
}
\description{
Generate extraC information for rxode2 models
}
\author{
Matthew L. Fidler
}
\keyword{internal}
