% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rudfui.R
\name{rxIntToBase}
\alias{rxIntToBase}
\title{Convert a positive  base}
\usage{
rxIntToBase(x, base = 36L)
}
\arguments{
\item{x}{integer to convert}

\item{base}{can be 2 to 36}
}
\value{
a sequence of letters and representing the number(s) input
}
\description{
Convert a positive  base
}
\examples{

rxIntToBase(1:100)

}
\author{
Matthew L. Fidler
}
