% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rudf.R
\name{.udfCallFunArg}
\alias{.udfCallFunArg}
\title{Get the function name with the current arguments as a string}
\usage{
.udfCallFunArg(fun, args)
}
\arguments{
\item{fun}{function name}

\item{args}{arguments}
}
\value{
string of the form 'fun(arg1, arg2)':
}
\description{
Get the function name with the current arguments as a string
}
\author{
Matthew L. Fidler
}
\keyword{internal}
