% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forder.R
\name{forderForceBase}
\alias{forderForceBase}
\title{Force using base order for rxode2 radix sorting}
\usage{
forderForceBase(forceBase = FALSE)
}
\arguments{
\item{forceBase}{boolean indicating if rxode2 should use R's
\code{\link[=order]{order()}} for radix sorting instead of
\code{data.table}'s parallel radix sorting.}
}
\value{
value of \code{forceBase} (can change if \code{data.table} is not
available)
}
\description{
Force using base order for rxode2 radix sorting
}
\examples{
\donttest{
forderForceBase(TRUE) # Use base `order` for rxode2 sorts
forderForceBase(FALSE) # Use `data.table` for rxode2 sorts
}
}
\keyword{internal}
