% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.rxTransInfo}
\alias{.rxTransInfo}
\title{Get the information about the rxode2 derived parameter transformation}
\usage{
.rxTransInfo(...)
}
\arguments{
\item{...}{Parameters translated, should be unquoted and not assigned to anything.}
}
\value{
Translation information; This list contains:
\itemize{
\item \verb{$str} A named string of the parameters as seen in the underlying C/C++
code. The parameters that are NA are not used in the linear
compartment model calculations.
\item \verb{$ncmt} the number of compartments in the model
\item \verb{$trans} the rxode2 translation number of the parameterization
}

This contains the linCmt()
translation number, the number of compartments and the parameters
}
\description{
Get the information about the rxode2 derived parameter transformation
}
\examples{

.rxTransInfo(cl, v , Vp, vp2, q, q2)

.rxTransInfo(k12, k21, k13, k31, kel, v)

.rxTransInfo(k12, k21, k13, k31, kel, v, ka)

.rxTransInfo(CL, V)

}
\author{
Matthew L. Fidler
}
\keyword{internal}
