% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{rxExpandSens2_}
\alias{rxExpandSens2_}
\title{Expand second order sensitivity}
\usage{
rxExpandSens2_(state, s1, s2)
}
\arguments{
\item{state}{is the state to expand}

\item{s1}{is the variables to calculate sensitivity}

\item{s2}{is the variables to calculate second order sensitivity}
}
\value{
string for symengine second order sensitivity
}
\description{
Expand second order sensitivity
}
\keyword{internal}
