% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxRaw.R
\name{rxGetDefaultSerialize}
\alias{rxGetDefaultSerialize}
\title{Get the Default Serialization Type}
\usage{
rxGetDefaultSerialize()
}
\value{
string indicating the default serialization type
}
\description{
Get the Default Serialization Type
}
\examples{

rxGetDefaultSerialize()

}
\author{
Matthew L. Fidler
}
